////////////////////////////////////////////////////////////////////////////////
//                            Appearance.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef APPEARANCE_H
#define APPEARANCE_H

#include "X3DTypes.h"
#include "X3DAppearanceNode.h"

namespace X3DTK {
namespace X3D {

class X3DMaterialNode;
class X3DTextureNode;
class X3DTextureTransformNode;
class LineProperties;
class FillProperties;

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">Shape 
/// component</a> defining an Appearance
/// (<a href="../../doc/X3DSpecs/Part01/components/Shape.html#Appearance">
/// documentation</a>).

class Appearance : public X3DAppearanceNode
{
public:
  /// Constructor.
  Appearance();
  /// Constructs a Material from its attributes.
  Appearance(const X3DMaterialNode *material, 
             const X3DTextureNode *texture, 
	     const X3DTextureTransformNode *textureTransform, 
	     const LineProperties *lineProperties, 
	     const FillProperties *fillProperties);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Appearance();
  
  /// Sets the Material attribute.
  void setMaterial(const X3DMaterialNode *material);
  /// Sets the Texture attribute.
  void setTexture(const X3DTextureNode *texture);
  /// Sets the TextureTransform attribute.
  void setTextureTransform(const X3DTextureTransformNode *textureTransform);
  /// Sets the LineProperties attribute.
  void setLineProperties(const LineProperties *lineProperties);
  /// Sets the FillProperties attribute.
  void setFillProperties(const FillProperties *fillProperties);

  /// Gets the Material attribute.  
  inline SFNode getMaterial() const {return _material;};
  /// Gets the Texture attribute.  
  inline SFNode getTexture() const {return _texture;};
  /// Gets the TextureTransform attribute.  
  inline SFNode getTextureTransform() const {return _textureTransform;};
  /// Gets the LineProperties attribute.  
  inline SFNode getLineProperties() const {return _lineProperties;};
  /// Gets the FillProperties attribute.  
  inline SFNode getFillProperties() const {return _fillProperties;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);

protected:  
  /// Copy constructor. 	     
  Appearance(const Appearance &A);

private:  
  /// Material node.
  SFNode _material;
  /// Texture node.
  SFNode _texture;
  /// Texture transform node.
  SFNode _textureTransform;
  /// Line properties.
  SFNode _lineProperties;
  /// Fill properties.
  SFNode _fillProperties;
};

}
}

#endif
