////////////////////////////////////////////////////////////////////////////////
//                            BboxUpdaterGlobalVariables.h                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef BBOXUPDATERGLOBALVARIABLES_H
#define BBOXUPDATERGLOBALVARIABLES_H

#include "GlobalVariables.h"
#include "Bbox.h"

#include <list>

namespace X3DTK {
namespace X3D {

/// GlobalVariables for the BboxUpdater module.

class BboxUpdaterGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  BboxUpdaterGlobalVariables();
  /// Destructor.
  virtual ~BboxUpdaterGlobalVariables();
  
  /// Finishes the traversal.
  virtual void finish();
  
  /// Sets the staticProcessing.
  void setStaticProcessing(bool value);
  /// Adds the Bounding Box BB.
  void addBbox(SFNode N, Bbox *BB);
  /// Sets the current Bbox (for the Geometry nodes of Shape).
  void setShapeBbox(const Bbox &BB);
  /// Adds the bounding Box to the list.
  void addBboxToMergeList(const Bbox &BB);
  
  /// Gets the staticProcessing value.
  inline bool getStaticProcessing() const {return _staticProcessing;};
  /// Gets the current Bbox.
  Bbox getShapeBbox() const {return _shapeBbox;};
  /// Gets the Bbox of N.
  Bbox *getBbox(SFNode N) const;
  /// Merges the Bbox of the list.
  Bbox mergeBbox();
  
private:
  bool _staticProcessing;
  Bbox _shapeBbox;
  std::list<std::pair<SFNode, Bbox *> > _BBList;
  std::list<Bbox> _BBToMergeList;  
};

}
}

#endif
