////////////////////////////////////////////////////////////////////////////////
//                            Box.h                                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef BOX_H
#define BOX_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining a Box 
/// (<a href="../../doc/X3DSpecs/Part01/components/geometry3D.html#Box">
/// documentation</a>).

class Box : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Box();
  /// Constructs a Box from \a size, the vector representing the diagonal of the Box. 
  /// The Box is centered on (0, 0, 0).
  Box(const SFVec3f &size);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Box();
  
  /// Sets the size attribute.
  void setSize(const SFVec3f &size);
  
  /// Gets the size attribute.
  inline const SFVec3f &getSize() const {return _size;};
  
  /// Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.
  Box(const Box &B);

private:  
  /// Size attribute.
  SFVec3f _size;
};

}
}

#endif
