////////////////////////////////////////////////////////////////////////////////
//                            Color.h                                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef COLOR_H
#define COLOR_H

#include "X3DTypes.h"
#include "X3DColorNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
/// Rendering component</a> defining a Color
/// (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#Color">
/// documentation</a>).

class Color : public X3DColorNode
{
public:
  /// Constructor.
  Color();
  /// Constructs a Color from its attributes.
  Color(const MFColor &color);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Color();
  
  /// Sets the Color attribute.
  void setColor(const MFColor &color);
  
  /// Gets the Color attribute.
  inline const MFColor &getColor() const {return _color;}; 
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;
  
protected:  
  /// Copy constructor.
  Color(const Color &C);

private:  
  /// Color attribute.
  MFColor _color;  
};

}
}

#endif
