////////////////////////////////////////////////////////////////////////////////
//                            ColorRGBA.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef COLORRGBA_H
#define COLORRGBA_H

#include "X3DTypes.h"
#include "X3DColorNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
/// Rendering component</a> defining a ColorRGBA
/// (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#ColorRGBA">
/// documentation</a>).

class ColorRGBA : public X3DColorNode
{
public:
  /// Constructor.
  ColorRGBA();
  /// Constructs a ColorRGBA from its attributes.
  ColorRGBA(const MFColorRGBA &color);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~ColorRGBA();
  
  /// Sets the Color attribute.
  void setColor(const MFColorRGBA &color);
  
  /// Gets the Color attribute.
  inline const MFColorRGBA &getColor() const {return _color;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;
  
protected:
  /// Copy constructor.
  ColorRGBA(const ColorRGBA &C);

private:  
  /// Color attribute.
  MFColorRGBA _color;  
};

}
}

#endif
