////////////////////////////////////////////////////////////////////////////////
//                            Component.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef COMPONENT_H
#define COMPONENT_H

#include "X3DTypes.h"

namespace X3DTK {

/// Class representing the concept of X3D component. It is a partition of the X3D 
/// nodes. It is implemented by a singleton per name.

class Component
{
  friend class Type;
  friend class X3DComponent;
  
public:
  /// Gets the component of name name.
  static Component *getComponent(const SFString &name);
  
  /// Returns the name of the component.
  inline SFString getName() const {return _name;};
  /// Gets the list of the types.
  inline MFType getTypeList() const {return _typeList;};

private:  
  Component(const SFString &name);
  ~Component();
  
  /// Adds one type.
  void addType(Type *type);
  /// Adds one reference.
  void addOneReference();
  /// Removes one reference of the Component.
  static void removeOneReference(Component *component);
  
  SFString _name;    
  int _refCount;
  MFType _typeList;
  
  static MFComponent _componentList;
};

}

#endif
