////////////////////////////////////////////////////////////////////////////////
//                            ConeDrawArray.h                                 //
////////////////////////////////////////////////////////////////////////////////

#ifndef CONEARRAY_H
#define CONEARRAY_H

#include "X3DTypes.h"
#include "X3DGLTypes.h"

#include <vector>
#include <utility>

namespace X3DTK {
namespace GL {

/// Class providing the arrays required by the opengl command drawElements() to draw
/// a Cone.

class ConeDrawArray
{
public:
  /// Gets the ConeDrawArray of section section.
  static ConeDrawArray *getInstanceOfSection(unsigned int section);
  /// Removes the ConeDrawArray of section section.
  void removeInstance();
  
  /// Gets the number of vertices of the side of the cone.
  unsigned int getConeSideSize() const;
  /// Gets the number of vertices of the side of the cone.
  unsigned int getConeBottomSize() const;

  /// Gets the address of the first element of the vertex array of the side.
  const void *getConeSideVertexArrayAddress() const;
  /// Gets the address of the first element of the vertex array of the bottom.
  const void *getConeBottomVertexArrayAddress() const;
  
  /// Gets the address of the first index of the side.  
  const unsigned int *getConeSideIndexArrayAddress() const;
  /// Gets the address of the first index of the bottom.
  const unsigned int *getConeBottomIndexArrayAddress() const;
 
private:
  unsigned int _section;
  
  /// Constructs the arrays for a cone of section vertices for the section.
  ConeDrawArray(unsigned int section);
  
  typedef struct 
  {
    unsigned int count;
    ConeDrawArray *ref;
  } data;
  
  typedef std::list<std::pair<unsigned int, data> > refList;

  static refList _refList;

  std::vector<N3F_V3F> _coneSideVertexArray;
  std::vector<N3F_V3F> _coneBottomVertexArray;
  std::vector<unsigned int> _coneSideIndexArray;
  std::vector<unsigned int> _coneBottomIndexArray;
};

}
}

#endif
