////////////////////////////////////////////////////////////////////////////////
//                            Coordinate.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef COORDINATE_H
#define COORDINATE_H

#include "X3DTypes.h"
#include "X3DCoordinateNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
/// Rendering component</a> defining a Coordinate
/// (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#Coordinate">
/// documentation</a>).

class Coordinate : public X3DCoordinateNode
{
public:
  /// Constructor.
  Coordinate();
  /// Constructs a Coordinate from its attributes.
  Coordinate(const MFVec3f &point);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Coordinate();
  
  /// Sets the Point attribute.
  void setPoint(const MFVec3f &point);
    
  /// Gets the Point attribute.
  inline const MFVec3f &getPoint() const {return _point;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;
  
protected: 
  /// Copy constructor.
  Coordinate(const Coordinate &C);

private:  
  /// Point attribute. 
  MFVec3f _point;  
};

}
}

#endif
