////////////////////////////////////////////////////////////////////////////////
//                            CylinderDrawArray.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef CYLINDERARRAY_H
#define CYLINDERARRAY_H

#include "X3DTypes.h"
#include "X3DGLTypes.h"

#include <vector>
#include <list>
#include <utility>

namespace X3DTK {
namespace GL {

/// Class providing the arrays required by the opengl command drawElements() to draw
/// a Cylinder. The arrays are implemented by singletons to avoid duplicating memory.

class CylinderDrawArray
{
public:
  /// Gets the CylinderDrawArray of section section.
  static CylinderDrawArray *getInstanceOfSection(unsigned int section);
  /// Removes the CylinderDrawArray of section section.
  void removeInstance();
  
  /// Gets the size of the bottom.
  unsigned int CylinderDrawArray::getCylinderBottomSize() const;
  /// Gets the size of the side.
  unsigned int CylinderDrawArray::getCylinderSideSize() const;
  /// Gets the size of the top.
  unsigned int CylinderDrawArray::getCylinderTopSize() const;
  /// Gets the address of the bottom vertex array.
  const void *CylinderDrawArray::getCylinderBottomVertexArrayAddress() const;
  /// Gets the address of the side vertex array.
  const void *CylinderDrawArray::getCylinderSideVertexArrayAddress() const;
  /// Gets the address of the top vertex array.
  const void *CylinderDrawArray::getCylinderTopVertexArrayAddress() const;
  /// Gets the address of the bottom index array.
  const unsigned int *CylinderDrawArray::getCylinderBottomIndexArrayAddress() const;
  /// Gets the address of the side index array.
  const unsigned int *CylinderDrawArray::getCylinderSideIndexArrayAddress() const;
  /// Gets the address of the top index array. 
  const unsigned int *CylinderDrawArray::getCylinderTopIndexArrayAddress() const;

private:
  unsigned int _section;
  
  CylinderDrawArray(unsigned int section); 

  typedef struct 
  {
    unsigned int count;
    CylinderDrawArray *ref;
  } data;
  
  typedef std::list<std::pair<unsigned int, data> > refList;

  static refList _refList;
  
  std::vector<N3F_V3F> _cylinderBottomVertexArray;
  std::vector<N3F_V3F> _cylinderSideVertexArray;
  std::vector<N3F_V3F>_cylinderTopVertexArray;
  std::vector<unsigned int> _cylinderBottomIndexArray;
  std::vector<unsigned int> _cylinderSideIndexArray;
  std::vector<unsigned int> _cylinderTopIndexArray;
};

}
}

#endif
