////////////////////////////////////////////////////////////////////////////////
//                            GLCone.h                                        //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLCONE_H
#define GLCONE_H

#include "X3DGLGeometry3DNode.h"

namespace X3DTK {
namespace GL {

class ConeDrawArray;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode
/// Cone.

class Cone : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Cone();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Cone();

  /// Sets the bottom radius attribute.
  void setBottomRadius(SFFloat bottomRadius);
  /// Sets the height attribute.
  void setHeight(SFFloat height);
  /// Sets the side attribute.
  void setSide(SFBool side);
  /// Sets the bottom attribute.
  void setBottom(SFBool bottom);
  /// Sets the cone array.
  void setConeArray(ConeDrawArray *coneArray);

  /// Gets the bottom radius attribute.
  inline SFFloat getBottomRadius() const {return _bottomRadius;};
  /// Gets the height attribute.
  inline SFFloat getHeight() const {return _height;};
  /// Gets the side attribute.
  inline SFBool getSide() const {return _side;};
  /// Gets the bottom attribute.
  inline SFBool getBottom() const {return _bottom;};
  /// Gets the cone array.
  inline ConeDrawArray *getConeArray() const {return _coneArray;};

  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;  
  
protected:
  /// Copy constructor.	   
  Cone(const Cone &C);
  
  /// Bottom radius attribute.
  SFFloat _bottomRadius;
  /// Height attribute.
  SFFloat _height;
  /// Side attribute.
  SFBool _side;
  /// Bottom attribute.
  SFBool _bottom;
  /// Cone array.
  ConeDrawArray *_coneArray;
};

}
}

#endif
