////////////////////////////////////////////////////////////////////////////////
//                            GLIndexedFaceSet.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLINDEXEDFACESET_H
#define GLINDEXEDFACESET_H

#include "X3DGLComposedGeometryNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// IndexedFaceSet.

class IndexedFaceSet : public X3DComposedGeometryNode
{
public:
  /// Constructor.
  IndexedFaceSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~IndexedFaceSet();

  /// Sets the N3F_V3F vertexArray.
  void setN3F_V3F_vertexArray(const std::vector<N3F_V3F> &N3F_V3FvertexArray);
  /// Sets the C4F_N3F_V3F vertexArray.
  void setC4F_N3F_V3F_vertexArray(const std::vector<C4F_N3F_V3F> &C4F_N3F_V3FvertexArray);
  /// Sets the T2F_N3F_V3F vertexArray.
  void setT2F_N3F_V3F_vertexArray(const std::vector<T2F_N3F_V3F> &T2F_N3F_V3F_vertexArray);
  /// Sets the T2F_C4F_N3F_V3F vertexArray.
  void setT2F_C4F_N3F_V3F_vertexArray(const std::vector<T2F_C4F_N3F_V3F> &T2F_C4F_N3F_V3F_vertexArray);
  /// Sets the indexFaceArray.
  void setIndexArray(const MFInt32 &indexArray);

  /// Gets the N3F_V3F vertexArray.
  inline std::vector<N3F_V3F> &N3F_V3F_vertexArray() {return _N3F_V3F_vertexArray;};
  /// Gets the C4F_N3F_V3F vertexArray.
  inline std::vector<C4F_N3F_V3F> &C4F_N3F_V3F_vertexArray() {return _C4F_N3F_V3F_vertexArray;};
  /// Gets the T2F_N3F_V3F vertexArray.
  inline std::vector<T2F_N3F_V3F> &T2F_N3F_V3F_vertexArray() {return _T2F_N3F_V3F_vertexArray;};
  /// Gets the T2F_C4F_N3F_V3F vertexArray.
  inline std::vector<T2F_C4F_N3F_V3F> &T2F_C4F_N3F_V3F_vertexArray() {return _T2F_C4F_N3F_V3F_vertexArray;};
  /// Gets the indexFaceArray.
  inline MFInt32 &indexArray() {return _indexArray;};
  /// Gets the X3DToGLIndex.
  inline const std::vector<MFInt32> &getX3DToGLIndex() {return _X3DtoGLIndex;};
  /// Computes the normals.
  virtual void computeNormals();

  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;
    
protected:
  /// Copy constructor.         
  IndexedFaceSet(const IndexedFaceSet &G);

private:   
  /// N3F_V3F vertexArray.
  std::vector<N3F_V3F> _N3F_V3F_vertexArray;
  /// C4F_N3F_V3F vertexArray.
  std::vector<C4F_N3F_V3F> _C4F_N3F_V3F_vertexArray;
  /// T2F_N3F_V3F vertexArray.
  std::vector<T2F_N3F_V3F> _T2F_N3F_V3F_vertexArray;
  /// T2F_C4F_N3F_V3F vertexArray.
  std::vector<T2F_C4F_N3F_V3F> _T2F_C4F_N3F_V3F_vertexArray;
  /// indexFaceArray.
  MFInt32 _indexArray;
  /// Relation between X3D vertex array and GL vertex array.
  std::vector<MFInt32> _X3DtoGLIndex;
  
  /// Empties the arrays.
  void emptyArrays();
  /// Computes the index array, tesselating the polygon by an ad-hoc method.
  void computeIndexArray(const MFInt32 &coordIndex);
  ///  Computes the vertex Array with no color and no texCoord.
  void computeNoColorNoTexCoord(const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex);
  /// Computes the vertex Array with color and no texCoord.
  void computeColorNoTexCoord(const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex);
  /// Computes the vertex Array with no color and texCoord.
  void computeNoColorTexCoord(const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex);
  /// Computes the vertex Array with color and texCoord.
  void computeColorTexCoord(const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex);
  /// Computes the arrays.
  void computeArrays(const MFVec3f &vertexArray, const MFInt32 &coordIndex, MFVec3f &normalArray);

  /// Computes the normals.
  void computeNoColorNoTexCoordNormals();
  /// Computes the normals.
  void computeColorNoTexCoordNormals();
  /// Computes the normals.
  void computeNoColorTexCoordNormals();
  /// Computes the normals.
  void computeColorTexCoordNormals();
  
};

}
}

#endif
