////////////////////////////////////////////////////////////////////////////////
//                            GLIndexedLineSet.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLINDEXEDLINESET_H
#define GLINDEXEDLINESET_H

#include "X3DGLGeometryNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// IndexedLineSet.

class IndexedLineSet : public X3DGeometryNode
{
public:
  /// Constructor.
  IndexedLineSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~IndexedLineSet();

  /// Sets the line width attribute.
  void setLineWidth(SFFloat lineWidth);
  /// Sets the verticesDuplicated attribute.
  void setVerticesDuplicated(SFBool verticesDuplicated);
  /// Sets the color attribute.
  void setColor(SFBool color);

  /// Sets the C4UB_V3F vertex array.
  void setC4UB_V3F_vertexArray(const std::vector<C4UB_V3F> &C4UB_V3F_vertexArray);   
  /// Sets the C4UB_V3F vertex array.
  void setV3F_vertexArray(const std::vector<SFVec3f> &V3F_vertexArray);
  /// Sets the index array.
  void setIndexArrayArray(const std::vector<MFInt32> &indexArrayArray);
  
  /// Gets the Line width attribute.
  inline SFFloat getLineWidth() const {return _lineWidth;};
  /// Gets the color per vertex attribute.
  inline SFBool getVerticesDuplicated() const {return _verticesDuplicated;};
  /// Gets the color attribute.
  inline SFBool getColor() const {return _color;};

  /// Gets the C4UB_V3F vertex array.
  inline std::vector<C4UB_V3F> &C4UB_V3F_vertexArray() {return _C4UB_V3F_vertexArray;};   
  /// Gets the C4UB_V3F vertex array.
  inline std::vector<SFVec3f> &V3F_vertexArray() {return _V3F_vertexArray;};
  /// Gets the index array.  
  inline std::vector<MFInt32> &indexArrayArray() {return _indexArrayArray;};

  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;  

protected:
  /// Copy constructor.      
  IndexedLineSet(const IndexedLineSet &I);
  
  /// Line width attribute.
  SFFloat _lineWidth;
  /// Color per vertex attribute.
  SFBool _verticesDuplicated;
  /// Color attribute.
  SFBool _color;
  /// Array of vertex arrays.
  std::vector<C4UB_V3F> _C4UB_V3F_vertexArray;
  /// Array of vertex arrays.
  std::vector<SFVec3f> _V3F_vertexArray;
  /// Array of Line index.
  std::vector<MFInt32> _indexArrayArray;
};

}
}

#endif
