////////////////////////////////////////////////////////////////////////////////
//                            GLMaterial.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLMATERIAL_H
#define GLMATERIAL_H

#include "X3DGLMaterialNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Material.

class Material : public X3DMaterialNode
{
public:
  /// Constructor.
  Material();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Material();
  
  /// Sets the diffuse color attribute.
  void setDiffuseColor(const MFFloat &diffuseColor);
  /// Sets the ambient color attribute.
  void setAmbientColor(const MFFloat &ambientColor);
  /// Sets the emissisve color attribute.
  void setEmissiveColor(const MFFloat &emissiveColor);
  /// Sets the shininess attribute.
  void setShininess(SFFloat shininess);
  /// Sets the specular color attribute.
  void setSpecularColor(const MFFloat &specularColor);

  /// Gets the diffuse color attribute.
  inline const MFFloat &getDiffuseColor() const {return _diffuseColor;};
  /// Gets the ambient color attribute.
  inline const MFFloat &getAmbientColor() const {return _ambientColor;};
  /// Gets the emissisve color attribute.
  inline const MFFloat &getEmissiveColor() const {return _emissiveColor;};
  /// Gets the shininess attribute.
  inline SFFloat getShininess() const {return _shininess;};
  /// Gets the specular color attribute.
  inline const MFFloat &getSpecularColor() const {return _specularColor;};

  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;  

protected:    
  /// Copy constructor.	   
  Material(const Material &M);

private:  
  /// Diffuse color attribute.
  MFFloat _diffuseColor;
  /// Ambient color attribute.
  MFFloat _ambientColor;
  /// Emissisve color attribute.
  MFFloat _emissiveColor;
  /// Shininess attribute.
  SFFloat _shininess;
  /// Specular color attribute.
  MFFloat _specularColor;
};

}
}

#endif
