////////////////////////////////////////////////////////////////////////////////
//                            GLSphere.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLSPHERE_H
#define GLSPHERE_H

#include "X3DGLGeometry3DNode.h"

namespace X3DTK {
namespace GL {

class SphereDrawArray;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Sphere.

class Sphere : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Sphere();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Sphere();
  
  /// Sets the radius attribute.
  void setRadius(SFFloat radius);
  /// Sets the sphere array.
  void setSphereDrawArray(SphereDrawArray *sphereArray);
  
  /// Gets the radius attribute.
  inline SFFloat getRadius() const {return _radius;};
  /// Gets the sphere array.
  inline SphereDrawArray *getSphereArray() const {return _sphereArray;};
  
  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const; 
  
protected:
  /// Copy constructor.	 
  Sphere(const Sphere &S);

private:  
  /// Radius attribute.
  SFFloat _radius;
  /// Sphere array.
  SphereDrawArray *_sphereArray;
};

}
}

#endif
