////////////////////////////////////////////////////////////////////////////////
//                            NodeCreationProxy.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef NODECREATIONPROXY_H
#define NODECREATIONPROXY_H

#include "X3DNodeProxy.h"
#include "Type.h"

namespace X3DTK {

class X3DComponentCreator;

/// Concrete class defining a NodeCreationProxy, creating X3D nodes from their names.

class NodeCreationProxy : public X3DNodeProxy
{
public:  
  /// Constructor.
  NodeCreationProxy();
  /// Destructor.
  virtual ~NodeCreationProxy();
  
  /// Sets the X3DComponentCreator.
  void setComponentCreator(X3DComponentCreator *component);
  /// Creates the SFNode from its name.
  X3D::SFNode createFromName(const SFString &name) const;
  /// Empties the list of component.
  void reset();
  
  /// Gets the name of the component of the type name.
  SFString getComponentNameOf(const SFString &name) const;

  friend NodeCreationProxy *joinNodeCreationProxy(NodeCreationProxy *N0, NodeCreationProxy *N1);
  
private:  
  /// Adds a type.
  virtual void addType(const Type *type);
  ///Gets the creation function related with name.
  CreationFunction *getCreationFunctionOf(const SFString &name) const;
  
  //Dictionary of all the creation functions used.
  CreationDict creationDict_;
  std::list<X3DComponentCreator *> componentList_;
};

}

#endif
