////////////////////////////////////////////////////////////////////////////////
//                            NodeVisitingProxy.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef NODEVISITINGPROXY_H
#define NODEVISITINGPROXY_H

#include "X3DAbstractNode.h"
#include "VisitingFunctions.h"
#include "EnterFunction.h"
#include "WalkOnFunction.h"
#include "LeaveFunction.h"
#include "X3DNodeProxy.h"

#include <list>

namespace X3DTK {

class Type;
class X3DComponentVisitor;

/// Concrete class defining a NodeProxy specialized in the visiting of nodes.

class NodeVisitingProxy : public X3DNodeProxy
{
public:  
  /// Constructor.
  NodeVisitingProxy();
  /// Destructor.
  virtual ~NodeVisitingProxy();
  
  /// Sets the X3DComponentCreator.
  void setComponentVisitor(X3DComponentVisitor *component);
  /// Enters the SFNode.
  inline void enter(SFAbstractNode N) const {visitingArray_[N->getType()->getId()]->enter(N);};
  /// Walks On the SFNode.
  inline bool walkOn(SFAbstractNode N, SFAbstractNode child = 0) const {return visitingArray_[N->getType()->getId()]->walkOn(N, child);};
  /// Leaves the SFNode.
  inline void leave(SFAbstractNode N) const {visitingArray_[N->getType()->getId()]->leave(N);};
  /// Empties the list of component.
  void reset();
  
  friend NodeVisitingProxy *joinNodeVisitingProxies(NodeVisitingProxy *N0, NodeVisitingProxy *N1);
  
private:  
  /// Adds a type.
  virtual void addType(const Type *type);
  /// finds the lowest functions in the derivation tree.
  EnterFunction *getEnterFunctionOf(const Type *type) const;
  WalkOnFunction *getWalkOnFunctionOf(const Type *type) const;
  LeaveFunction *getLeaveFunctionOf(const Type *type) const;
  
  //Dictionary of all the creation functions used.
  VisitingArray visitingArray_;
  std::list<X3DComponentVisitor *> componentList_;
};

}

#endif
