////////////////////////////////////////////////////////////////////////////////
//                            SceneSaverGlobalVariables.h                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef SCENESAVERGLOBALVARIABLES_H
#define SCENESAVERGLOBALVARIABLES_H

#include "GlobalVariables.h"
#include "NameDict.h"

#include <list>

namespace X3DTK {
namespace X3D {

/// GlobalVariables for the SceneSaver module.

class SceneSaverGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  SceneSaverGlobalVariables();
  /// Destructor.
  virtual ~SceneSaverGlobalVariables();
  
  /// Sets the hasUSE variable.
  void pushHasUSE(bool value);
  
  /// New line.
  void newLine();
  /// Pushes line.
  void pushCurrentLine();
  /// Writes the string.
  void writeCurrentLine(const SFString &string);
  /// Writes the string.
  void writeOpeningLine(const SFString &string);

  /// Adds a tabulation.
  void addTab();
  /// Removes a tabulation.
  void removeTab();
  
  /// Gets the SFString of the output.
  inline MFString getOutput() const {return _output;};
  /// Returns TRUE if the current visited node has a child.
  bool hasChild();
  /// Pops the line stack.
  void popLine();
  /// Returns the hasUSE variable.
  inline bool getHasUSE() const {return _hasUSEStack.front();};
  /// Pops the hasUSE stack.
  void popHasUSE();
  /// Gets the tabulation.
  inline SFString getTab() const {return _tab;};
  /// returns the DEF attribute of the node.
  SFString getDEF(const SFNode &N);
  /// returns the USE attribute of the node.
  SFString getUSE(const SFNode &N) const;
  
private:
  int _lineNumber;
  MFString _output; 
  SFString _tab;
  std::list<int> _lineStack;
  std::list<bool> _hasUSEStack;
  std::list<SFString> _DEFList;
};

}
}

#endif
