////////////////////////////////////////////////////////////////////////////////
//                            Shape.h                                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef SHAPE_H
#define SHAPE_H

#include "X3DTypes.h"
#include "X3DShapeNode.h"
#include "X3DBoundedObject.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">Shape 
/// component</a> defining a Shape
/// (<a href="../../doc/X3DSpecs/Part01/components/Shape.html#Shape">
/// documentation</a>).

class Shape : public X3DShapeNode, public X3DBoundedObject
{
public:
  /// Constructor.
  Shape();
  /// Constructs a Shape from its attributes.
  Shape(const X3DGeometryNode *geometry, 
        const X3DAppearanceNode *appearance, 
        const SFVec3f &bboxCenter, 
        const SFVec3f &bboxSize);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Shape();
  
  /// loads the attributes of the node.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor. 	
  Shape(const Shape &S);
};

}
}

#endif
