////////////////////////////////////////////////////////////////////////////////
//                            StaticGroup.h                                   //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef STATICGROUP_H
#define STATICGROUP_H

#include "X3DTypes.h"
#include "X3DChildNode.h"
#include "X3DBoundedObject.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/group.html">
/// Grouping component</a> defining a StaticGroup
/// (<a href="../../doc/X3DSpecs/Part01/components/grouping.html#StaticGroup">
/// documentation</a>).

class StaticGroup : public X3DChildNode, public X3DBoundedObject
{
public:
  /// Constructor.
  StaticGroup();
  /// Constructs a StaticGroup from its attributes.
  StaticGroup(const SFVec3f &bboxCenter, 
	      const SFVec3f &bboxSize);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~StaticGroup();
  
  /// Adds a child to the Children attribute. Return TRUE if succeeds.
  bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child. Return TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return _children;};
  /// Loads the attributes from a FileElement.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected: 
  /// Copy constructor.		
  StaticGroup(const StaticGroup &G);

private:    
  /// Children attribute.
  MFNode _children;  
};

}
}

#endif
