////////////////////////////////////////////////////////////////////////////////
//                            TemplateEdge.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEEDGE_H
#define TEMPLATEEDGE_H

#include "MeshTypes.h"
#include "TemplateEdgeContent.h"

namespace X3DTK {
namespace Mesh {

template<class A, class B, class C> class TemplateVertex;
template<class A, class B, class C> class TemplateFace;

/// Class providing a template edge for the Mesh.

template<class EData, class FData, class VData>
class TemplateEdge
{
  template<class A, class B, class C, class D> friend class TemplateMesh;
  template<class A, class B, class C> friend class TemplateDirectEdge;
  template<class A, class B, class C> friend class TemplateIndirectEdge;
public:
  /// Returns the symetric edge.
  inline TemplateEdge *const getSymetric() {return _symetric;};
  /// Returns the Vertex from.
  inline TemplateVertex<VData, EData, FData> *getFromVertex() const {return (_direct ? _edgeContent->getFromVertex() : _edgeContent->getToVertex());};
  /// Gets the Vertex to.
  inline TemplateVertex<VData, EData, FData> *getToVertex() const {return (_direct ? _edgeContent->getToVertex() : _edgeContent->getFromVertex());};
  /// Gets the left faces.
  inline const MTemplateFace<FData, VData, EData> &getLeftFaces() const {return (_direct ? _edgeContent->get1Faces() : _edgeContent->get2Faces());};
  /// Gets the right faces.
  inline const MTemplateFace<FData, VData, EData> &getRightFaces() const {return (_direct ? _edgeContent->get2Faces() : _edgeContent->get1Faces());};
  /// Gets the faces.
  MTemplateFace<FData, VData, EData> getFaces() const 
  {
    MTemplateFace<FData, VData, EData> faces;
    const MTemplateFace<FData, VData, EData> &r1faces = _edgeContent->get1Faces();
    const MTemplateFace<FData, VData, EData> &r2faces = _edgeContent->get2Faces();
    
    //faces.reserve(r1faces.size() + r2faces.size() + 1);
    for (typename MTemplateFace<FData, VData, EData>::const_iterator it = r1faces.begin(); it != r1faces.end(); ++it)
    {
      faces.back() = *it;
      faces.push_back((TemplateFace<FData, VData, EData> *)(0x1));
    }  
    for (typename MTemplateFace<FData, VData, EData>::const_iterator it = r2faces.begin(); it != r2faces.end(); ++it)
    {
      faces.back() = *it;
      faces.push_back((TemplateFace<FData, VData, EData> *)(0x1));
    }  

    return faces;
  };
  /// Returns true if is a boundary.
  inline bool isBoundary() const {return _edgeContent->isBoundary();};
  /// Gets the data.
  inline EData &data() {return _edgeContent->data();};
  /// Gets the data.
  inline const EData &data() const {return _edgeContent->data();};

protected:  
  void setDirect(bool value) {_direct = value;};

private:
  TemplateEdgeContent<EData, FData, VData> *const _edgeContent;
  bool _direct;
  TemplateEdge *_symetric;
  
  TemplateEdge(TemplateEdgeContent<EData, FData, VData> *const edgeContent)
  : _edgeContent(edgeContent), _symetric(0)
  {};

  
  void setSymetric(TemplateEdge *symetric) {_symetric = symetric;};
  
  /// Adds a left face.
  void addLeftFace(TemplateFace<FData, VData, EData> *face) 
  {
    if (_direct)
      _edgeContent->add1Face(face);
    else
      _edgeContent->add2Face(face);    
  };
  /// Adds a right face.
  void addRightFace(TemplateFace<FData, VData, EData> *face) 
  {
    if (_direct)
      _edgeContent->add2Face(face);
    else
      _edgeContent->add1Face(face);    
  };
};


}
}

#endif
