////////////////////////////////////////////////////////////////////////////////
//                            TemplateFace.h                                   //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEFACE_H
#define TEMPLATEFACE_H

#include "MeshTypes.h"

#include <vector>
#include <iostream>

namespace X3DTK {
namespace Mesh {

template<class A, class B, class C> class TemplateEdge;

/// Template class providing a Face for the Mesh structure.

template<class FData, class VData, class EData>
class TemplateFace
{
  template<class A, class B, class C, class D> friend class TemplateMesh;
public:
  /// Gets the index.
  inline unsigned int getIndex() const {return _index;};
  /// Gets the edges of the faces. The order is given b the order of definition of the face.
  inline const MTemplateEdge<EData, FData, VData> &getEdges() const {return _edges;};
  /// Gets the data.
  inline FData &data() {return _data;};
  /// Gets the data.
  inline const FData &data() const {return _data;};

private:
  const unsigned int _index;
  MTemplateEdge<EData, FData, VData> _edges;
  FData _data;
  
  /// Constructor.
  TemplateFace(unsigned int i, const MTemplateEdge<EData, FData, VData> &edges)
  : _index(i), _edges(edges)
  {
    if (edges.size() == 0)
      std::cerr << "Mesh::TemplateFace : cannot create a face of 0 edge!" << std::endl;
  };
};

}
}

#endif
