////////////////////////////////////////////////////////////////////////////////
//                            TemplateMeshBuilder.h                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESHBUILDER_H
#define TEMPLATEMESHBUILDER_H

#include "MeshTypes.h"
#include "X3DOnePassProcessor.h"
#include "TemplateMeshBuilderGlobalVariables.h"
#include "DefaultDFS.h"
#include "TemplateMeshBuilderCoreVisitor.h"
#include "TemplateMeshBuilderGeometry3DVisitor.h"
#include "TemplateMeshBuilderGroupingVisitor.h"
#include "TemplateMeshBuilderShapeVisitor.h"
#include "TemplateMeshBuilderRenderingVisitor.h"
#include "Scene.h"
#include "MeshScene.h"
#include "MeshAlgoFunctions.h"

namespace X3DTK {
namespace X3D {

/// Processor defining a builder of template meshes.

template<class MData, class VData, class EData, class FData>
class TemplateMeshBuilder : public X3DOnePassProcessor
{     
public:
  /// Constructor. 
  TemplateMeshBuilder() 
  {
    globalVariables = GVManager::getInstanceOf<TemplateMeshBuilderGlobalVariables>();
    dfs = new DefaultDFS();
  
    dfs->setComponentVisitor(new TemplateMeshBuilderCoreVisitor());
    dfs->setComponentVisitor(new TemplateMeshBuilderGeometry3DVisitor<MData, VData, EData, FData>());
    dfs->setComponentVisitor(new TemplateMeshBuilderGroupingVisitor());
    dfs->setComponentVisitor(new TemplateMeshBuilderShapeVisitor());
    dfs->setComponentVisitor(new TemplateMeshBuilderRenderingVisitor());
  };
  /// Virtual destructor.
  virtual ~TemplateMeshBuilder() 
  {
    delete dfs;
  };
  /// Sets the component visitor.
  void setComponentVisitor(X3DComponentVisitor *component)
  {
    dfs->setComponentVisitor(component);
  };
    
  /// Builds the mesh from N.
  virtual Mesh::SFNode build(SFNode N)
  {
    std::cout << "building mesh" << std::endl;
    globalVariables->init();
    dfs->traverse(N);
    globalVariables->finish();
    
    return globalVariables->getRoot();
  };
  
  /// Builds the mesh scene from S.
  Mesh::Scene *build(Scene *S)
  {
    return static_cast<Mesh::Scene *>(build(static_cast<SFNode>(S)));
  };
  
private:
  TemplateMeshBuilderGlobalVariables *globalVariables;
  DFS *dfs; 
};

}
}

#endif
