////////////////////////////////////////////////////////////////////////////////
//                            TemplateMeshBuilderGeometry3DVisitor.h          //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEMESHBUILDERGEOMETRY3DVISITOR_H
#define TEMPLATEMESHBUILDERGEOMETRY3DVISITOR_H

#include "Geometry3DVisitor.h"
#include "TemplateMeshBuilderGlobalVariables.h"
#include "Coordinate.h"
#include "Box.h"
#include "Cone.h"
#include "Cylinder.h"
#include "IndexedFaceSet.h"
#include "Sphere.h"
#include "TemplateMesh.h"
#include "TemplateMeshBuilder.h"
#include "MeshAlgoFunctions.h"

namespace X3DTK {
namespace X3D {

/// Visitor for the Geometry3D component of the TemplateMeshBuilder processor.

template<class MData, class VData, class EData, class FData>
class TemplateMeshBuilderGeometry3DVisitor : public Geometry3DVisitor
{
public:
  /// Constructor.
  TemplateMeshBuilderGeometry3DVisitor()
  : Geometry3DVisitor()
  {
    // Enter functions.
    defineNewEnterFunction<TemplateMeshBuilderGeometry3DVisitor, IndexedFaceSet>(&TemplateMeshBuilderGeometry3DVisitor::enterIndexedFaceSet);

    // GlobalVariables assignation.
    globalVariables = GVManager::getInstanceOf<TemplateMeshBuilderGlobalVariables>();
  };
  
  /// Destructor.
  virtual ~TemplateMeshBuilderGeometry3DVisitor()
  {
  };

  /// Enters a IndexedFaceSet.
  virtual void enterIndexedFaceSet(IndexedFaceSet *I) const
  {
    // future optim? reserve the vertices and edges.
  
    typedef Mesh::TemplateVertex<VData, EData, FData> Vertex;
    typedef Mesh::MTemplateVertex<VData, EData, FData> MVertex;
    typedef Mesh::TemplateEdge<EData, FData, VData> Edge;
    typedef Mesh::MTemplateEdge<EData, FData, VData> MEdge;
    typedef Mesh::TemplateFace<FData, VData, EData> Face;
    typedef Mesh::MTemplateFace<FData, VData, EData> MFace;
           
    Mesh::TemplateMesh<MData, VData, EData, FData> *TM = new Mesh::TemplateMesh<MData, VData, EData, FData>();
    globalVariables->getTop()->addChild(TM);
    globalVariables->pushNode(TM);
    
    // filling the mesh.
    // points creation
    Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
    if (CoordNode == NULL)
      return;
    
    MFVec3f pointArray = CoordNode->getPoint();
    if (pointArray.empty())
      return;
    
    Mesh::start<IndexedFaceSet, Mesh::TemplateMesh<MData, VData, EData, FData> >(I, TM);
    
    // iterating the points
    for (MFVec3f::const_iterator itPoint = pointArray.begin(); itPoint != pointArray.end(); ++itPoint)
      TM->createVertex();
    
    SFBool ccw = I->getCcw();
    
    //iterating the faces
    const MFInt32 &coordIndex = I->getCoordIndex();
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    
    while (itCoordIndex != coordIndex.end())
    {
      Vertex *A, *B;      
      Edge *E;
      // creating the edges of the face.
      Vertex *start = TM->getVertices()[*itCoordIndex];
      
      A = start;
      ++itCoordIndex;

      std::vector<Edge *> faceEdgeList;      
      while (*itCoordIndex != -1)
      {
        B = TM->getVertices()[*itCoordIndex];
        ++itCoordIndex;
      
        E = TM->getEdge(A, B);
        if (E == 0)
          E = TM->createEdge(A, B);
            
        faceEdgeList.push_back(E);
        // moving to the next edge.
        A = B;
      }
      // ending the loop
      B = start;
      E = TM->getEdge(A, B);
      if (E == 0)
        E = TM->createEdge(A, B);
  
      faceEdgeList.push_back(E);
        
      // creating a new face.
      MEdge me(faceEdgeList, ccw);
      
      Face *F = TM->createFace(me);
      
      // processing the new face.
      Mesh::processNewFace<IndexedFaceSet, Face, Mesh::TemplateMesh<MData, VData, EData, FData> >(I, F, TM);

      ++itCoordIndex;
    }
    
    // ending the algorithm.
    Mesh::end<IndexedFaceSet, Mesh::TemplateMesh<MData, VData, EData, FData> >(I, TM);
  };
  
protected:
  TemplateMeshBuilderGlobalVariables *globalVariables;
};

}
}

#endif
