////////////////////////////////////////////////////////////////////////////////
//                            TemplateVertex.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEVERTEX_H
#define TEMPLATEVERTEX_H

#include "MeshTypes.h"

#include <vector>

namespace X3DTK {
namespace Mesh {

template<class A, class B, class C> class TemplateEdge;
template<class A, class B, class C> class TemplateFace;

/// Template class providing a Vertex class for the Mesh structure. This class can only
/// been constructed by a TemplateMesh.

template<class VData, class EData, class FData>
class TemplateVertex
{
  template<class A, class B, class C, class D> friend class TemplateMesh;
public:
  /// Gets the index.
  inline unsigned int getIndex() const {return _index;};
  /// Gets the rounding edges.
  inline const MTemplateEdge<EData, FData, VData> &getEdges() const {return _edges;};
  /// Gets the rounding faces.
  inline const MTemplateFace<FData, VData, EData> &getFaces() const {return _faces;};
  /// Gets the data.
  inline VData &data() {return _data;};
  /// Gets the data.
  inline const VData &data() const {return _data;};
  
private:
  const unsigned int _index;
  MTemplateEdge<EData, FData, VData> _edges;
  MTemplateFace<FData, VData, EData> _faces;
  VData _data;

  /// Constructor.
  TemplateVertex(unsigned int i)
  : _index(i) {};
  /// Adds an edge.
  void addEdge(TemplateEdge<EData, FData, VData> *E) {_edges.back() = E; _edges.push_back((TemplateEdge<EData, FData, VData> *)(0x1));};
  /// Adds a face.
  void addFace(TemplateFace<FData, VData, EData> *F) {_faces.back() = F; _faces.push_back((TemplateFace<FData, VData, EData> *)(0x1));};
};

}
}

#endif
