////////////////////////////////////////////////////////////////////////////////
//                            Transform.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRANSFORM_H
#define TRANSFORM_H

#include "X3DTypes.h"
#include "X3DGroupingNode.h"
#include "X3DBoundedObject.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/group.html">
/// Grouping component</a> defining a Transform
/// (<a href="../../doc/X3DSpecs/Part01/components/grouping.html#Transform">
/// documentation</a>).

class Transform : public X3DGroupingNode, public X3DBoundedObject
{
public:  
  /// Constructor.
  Transform();
  /// Constructs a Transform from its attributes.
  Transform(const SFVec3f &center, 
	    const SFRotation &rotation, 
	    const SFVec3f &scale, 
	    const SFRotation &scaleOrientation, 
	    const SFVec3f &translation, 
	    const SFVec3f &bboxCenter, 
	    const SFVec3f &bboxSize);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Transform();

  /// Sets the center of the transformation.
  void setCenter(const SFVec3f &center);
  /// Sets the rotation of the transformation.
  void setRotation(const SFRotation &rotation);
  /// Sets the scale of the transformation.
  void setScale(const SFVec3f &scale);
  /// Sets the scale orientation of the transformation.
  void setScaleOrientation(const SFRotation &scaleOrientation);
  /// Sets the translation of the transformation.
  void setTranslation(const SFVec3f &translation);

  /// Gets the center of the transformation.
  inline const SFVec3f &getCenter() const {return _center;};
  /// Gets the rotation of the transformation.
  inline const SFRotation &getRotation() const {return _rotation;};
  /// Gets the scale of the transformation.
  inline const SFVec3f &getScale() const {return _scale;};
  /// Gets the scale orientation of the transformation.
  inline const SFRotation &getScaleOrientation() const {return _scaleOrientation;};
  /// Gets the translation of the transformation.
  inline const SFVec3f &getTranslation() const {return _translation;};

  /// Loads the attributes from a X3DFileElement *.
  void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.
  Transform(const Transform &T);

private:
  /// Center attribute.
  SFVec3f _center;
  /// Rotation attribute.
  SFRotation _rotation;
  /// Scale attribute.
  SFVec3f _scale;
  /// Scale orientation attribute.
  SFRotation _scaleOrientation;
  /// Translation attribute.
  SFVec3f _translation;
};

}
}

#endif
