////////////////////////////////////////////////////////////////////////////////
//                            Type.h                                          //
////////////////////////////////////////////////////////////////////////////////

#ifndef TYPE_H
#define TYPE_H

#include "X3DTypes.h"

namespace X3DTK {

class Component;
class X3DNodeProxy;

/// Class defining an artificial data structure storing the derivation tree. It is
/// based upon singleton for tag. It means that it exists only one instance of Type 
/// of tag name "Cylinder". 

class Type
{
  friend class NodeCreationProxy;
  friend class NodeWalkingProxy;
  friend class NodeVisitingProxy;
  friend class X3DComponentCreator;
  friend class X3DComponentWalker;
  friend class X3DComponentVisitor;
  friend class X3DAbstractNode;

public:
  /// Gets the Type of name name.
  static Type *getTypeOfName(const SFString &component, const SFString &name);
  /// Gets the Type of id id.
  static Type *getTypeOfId(int id);
  /// Recomputes the id and changes all the nodeProxy array.(Not yet implemented)
  static void recomputeIds();
  
  /// Returns the name of the node
  inline SFString getName() const {return _name;};
  /// Return the id of the node.
  inline int getId() const {return _id;};
  /// Gets the parent.
  inline Type *getParent() const {return _parent;};
  /// Gets the list of the children.
  inline MFType getChildList() const {return _childList;};
  /// Gets the component.
  inline Component *getComponent() const {return _component;};
  /// Prints the current derivation tree.
  static void printDerivationTree();
    
private:
  /// Constructor.
  Type(const SFString &name);
  ~Type();

  /// Sets the parent of the derivation hierarchy.
  void setParent(const Type *parent);
  /// Adds one instance of the type.
  void addOneReference();
  /// Adds one instance of the type and of the parent types.
  void addOneReferenceRecursive();
  /// Removes one instance of the type.
  static void removeOneReference(Type *type);
  
  /// Defines a new Tag name. Only X3DNode is able to call this method.
  static void defineTypeName(Type **type, const SFString &name, const SFString &component);
  /// Adds a new proxy. Only X3DNodeProxy is able to call this method.
  static void addNodeProxy(X3DNodeProxy *proxy);
  /// Removes proxy from the list. Only X3DNodeProxy is able to call this method.
  static void removeNodeProxy(X3DNodeProxy *proxy);

  /// Specifies to Type that the next new Nodes are necessary to get their name and 
  /// will not be used then.
  static void beginFunctionDefinition();
  /// Specifies to Type that the function definition is terminated.
  static void endFunctionDefinition();

  
  /// attributes
  SFString _name;
  int _id;
  int _refCount;
  Type *_parent;
  MFType _childList;
  Component *_component;
  
  /// static attributes
  static int count;
  static MFType _typeList;  
  static MFNodeProxy _proxyList;
  static bool _functionDefinition;
 
  void printDerivationTree(SFString tab) const;
};

}

#endif
