////////////////////////////////////////////////////////////////////////////////
//                            VisitingFunctions.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef VISITINGFUNCTIONS_H
#define VISITINGFUNCTIONS_H

#include "EnterFunction.h"
#include "WalkOnFunction.h"
#include "LeaveFunction.h"

namespace X3DTK {

/// Concrete class defining the concept of visiting functions, by grouping the 3 
/// visiting functions: enter, walkOn, leave.

class VisitingFunctions
{
public:
  /// Constructor.
  VisitingFunctions(const EnterFunction *enter, const WalkOnFunction *walkon, const LeaveFunction *leave);
  /// Destructor.
  ~VisitingFunctions();
  
  /// Sets the enter function.
  void setEnterFunction(const EnterFunction *enter);
  /// Sets the walkOn function.
  void setWalkOnFunction(const WalkOnFunction *walkon);
  /// Sets the leave function.
  void setLeaveFunction(const LeaveFunction *leave);

  /// Enter the node.
  inline void enter(SFAbstractNode N) const {enter_->enter(N);};
  /// continue the node.
  inline bool walkOn(SFAbstractNode N, SFAbstractNode child = 0) const {return walkon_->walkOn(N, child);};  
  /// leave the node.
  inline void leave(SFAbstractNode N) const  {leave_->leave(N);};

private:
  EnterFunction *enter_;
  WalkOnFunction *walkon_;
  LeaveFunction *leave_;
};

}

#endif
