////////////////////////////////////////////////////////////////////////////////
//                            X3DBoundedObject.h                              //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DBOUNDEDOBJECT_H
#define X3DBOUNDEDOBJECT_H

#include "X3DTypes.h"
#include "X3DObject.h"
#include "X3DFileElement.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/group.html">
/// Grouping component</a> defining an X3DBoundedObject
/// (<a href="../../doc/X3DSpecs/Part01/components/grouping.html#X3DBoundedObject">
/// documentation</a>).

class X3DBoundedObject : public X3DObject
{
public:
  /// Constructor.
  X3DBoundedObject();
  /// Constructs an X3DBoundedObject from its attributes.
  X3DBoundedObject(const SFVec3f &bboxCenter,
                   const SFVec3f &bboxSize);
		   
  /// Copy constructor.
  X3DBoundedObject(const X3DBoundedObject &N);

  /// Virtual destructor.
  virtual ~X3DBoundedObject();

  /// Sets the BboxCenter attribute.
  void setBboxCenter(const SFVec3f &bboxCenter);
  /// Sets the BboxSize attribute.
  void setBboxSize(const SFVec3f &bboxSize);

  /// Gets the BboxCenter attribute.
  inline const SFVec3f &getBboxCenter() const {return _bboxCenter;};
  /// Gets the BboxSize attribute.
  inline const SFVec3f &getBboxSize() const {return _bboxSize;};

  /// Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

private:
  /// Center of the Bounding Box.
  SFVec3f _bboxCenter;
  /// Size of the Bounding Box.
  SFVec3f _bboxSize; 
};

}
}

#endif
