////////////////////////////////////////////////////////////////////////////////
//                            X3DComponentCreator.h                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPONENTCREATOR_H
#define X3DCOMPONENTCREATOR_H

#include "X3DComponent.h"
#include "CreationFunction.h"

namespace X3DTK {

/// Abstract class defining the concept of component creator.

class X3DComponentCreator : public X3DComponent
{
public:
  /// Constructor.
  X3DComponentCreator();
  /// Destructor.
  virtual ~X3DComponentCreator() = 0;
  
  /// Gets the creation of name.
  CreationFunction *getCreationFunctionOf(const SFString &name) const;
  /// Returns the list of functions
  inline CreationDict getCreationDict() const {return creationDict_;};
  
  /// Returns TRUE if the component contains the type name.
  bool contains(const SFString &name) const;
  
protected:  
  /// Define a new creation function.
  template<class T> void defineNewCreationFunction(const SFString &name, X3D::SFNode (T::*ptrF)() const)
  {
    CreationFunction *CF = new CreationFunction(reinterpret_cast<ptrToCreationFunction>(ptrF), this);
    creationDict_.insert(std::pair<SFString, CreationFunction *>(name, CF));
  };
  
//private:
  CreationDict creationDict_;
};

}

#endif
