////////////////////////////////////////////////////////////////////////////////
//                            X3DGLComposedGeometryNode.h                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DGLCOMPOSEDGEOMETRYNODE_H
#define X3DGLCOMPOSEDGEOMETRYNODE_H

#include "X3DGLGeometry3DNode.h"

#include <GL/gl.h>

namespace X3DTK {
namespace GL {

/// Class providing an interface for the equivalent of the X3DComposedGeometryNode in the X3DGL tree.

class X3DComposedGeometryNode : public X3DGeometry3DNode
{
public:
  /// Constructor.
  X3DComposedGeometryNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DComposedGeometryNode();

  /// Sets the ccw attribute.
  void setCcw(SFBool ccw);
  /// Sets the solid attribute.
  void setSolid(SFBool solid);
  /// Sets the vertices duplicated attribute.
  void setVerticesDuplicated(SFBool verticesDuplicated);
  /// Sets the color attribute.
  void setColor(SFBool color);
  /// Sets the texCoord attribute.
  void setTexCoord(SFBool texCoord);
  /// Sets the normalPerVertex attribute.  
  void setNormalPerVertex(SFBool normalPerVertex);
  /// Sets the vertex format.
  void setVertexFormat(GLenum format);

  /// gets the ccw attribute.
  inline SFBool getCcw() const {return _ccw;};
  /// Gets the solid attribute.
  inline SFBool getSolid() const {return _solid;};
  /// Gets the vertices duplicated attribute.
  inline SFBool getVerticesDuplicated() const {return _verticesDuplicated;};
  /// Gets the color attribute.
  inline SFBool getColor() const {return _color;};
  /// Gets the texCoord attribute.
  inline SFBool getTexCoord() const {return _texCoord;};
  /// Gets the normalPerVertex attribute.  
  inline SFBool getNormalPerVertex() {return _normalPerVertex;};
  /// Gets the vertex format.
  inline GLenum getVertexFormat() const {return _format;};
  
protected:
  /// Copy constructor.
  X3DComposedGeometryNode(const X3DComposedGeometryNode &N);

private:  
  /// Ccw attribute.
  SFBool _ccw;
  /// Solid attribute.
  SFBool _solid;
  /// Vertices duplicated attribute.
  SFBool _verticesDuplicated;
  /// Color attribute.
  SFBool _color;
  /// TexCoord attribute.
  SFBool _texCoord;
  /// NormalPerVertex.
  SFBool _normalPerVertex;
  /// vertex format.
  GLenum _format;
};

}
}

#endif
