////////////////////////////////////////////////////////////////////////////////
//                            X3DGLNode.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DGLNODE_H
#define X3DGLNODE_H

#include "X3DAbstractNode.h"
#include "X3DNode.h"
#include "X3DGLTypes.h"

#include <GL/gl.h>

namespace X3DTK {
namespace GL {

/// Base class of the Core component. A set of X3DGLNode represents a scene graph
/// related with the X3D scene graph. Each X3DNode able to be drawn directly has 
/// its X3DGLNode correspondence. For example X3DIndexedFaceSet is related with 
/// GLIndexedFaceSet, whereas Coordinate has no X3DGLNode correspondent. 

class X3DNode : public X3DAbstractNode
{
public:
  /// Constructor.
  X3DNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DNode();
  
  /// Sets the x3dReference.
  void setX3DReference(X3D::SFNode N);
  /// Returns the X3DNode equivalent.
  inline X3D::SFNode getX3DReference() const {return x3dReference;};
  
  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void render() const;  
  
protected:
  /// Copy constructor.
  X3DNode(const X3DNode &N);
  
  /// Pointer to the X3DNode equivalent.
  X3D::SFNode x3dReference;
};

}
}

#endif
