////////////////////////////////////////////////////////////////////////////////
//                            X3DGLShapeNode.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DGLSHAPENODE_H
#define X3DGLSHAPENODE_H

#include "X3DGLChildNode.h"

namespace X3DTK {
namespace GL {

class X3DGeometryNode;
class X3DAppearanceNode;

/// Class providing an interface for the equivalent of the X3DShapeNode in the X3DGL tree.

class X3DShapeNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DShapeNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DShapeNode();
  
  /// Sets the Geometry attribute.
  void setGeometry(const X3DGeometryNode *geometry);
  /// Sets the Appearance attribute.
  void setAppearance(const X3DAppearanceNode *appearance);

  /// Gets the Geometry attribute.
  inline SFNode getGeometry() const {return _geometry;};
  /// Gets the Appearance attribute.
  inline SFNode getAppearance() const {return _appearance;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFAbstractNode &N);
  
protected:
  /// Copy constructor.	       
  X3DShapeNode(const X3DShapeNode &N);

private:  
  /// Geometry node.
  SFNode _geometry;
  /// Appearance node.
  SFNode _appearance;  
};

}
}

#endif
