////////////////////////////////////////////////////////////////////////////////
//                            X3DGroupingNode.h                               //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DGROUPINGNODE
#define X3DGROUPINGNODE

#include "X3DTypes.h"
#include "X3DChildNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/group.html">
/// Grouping component</a> defining an X3DGroupingNode
/// (<a href="../../doc/X3DSpecs/Part01/components/grouping.html#X3DGroupingNode">
/// documentation</a>).

class X3DGroupingNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DGroupingNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DGroupingNode();

  /// Adds a child to the Children attribute. Returns TRUE if succeeds.
  bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child. Return TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return _children;};
  
protected:
  /// Copy constructor.		  
  X3DGroupingNode(const X3DGroupingNode &N);

private:  
  /// Children attribute.
  MFNode _children;
};

}
}

#endif
