////////////////////////////////////////////////////////////////////////////////
//                            X3DLoader.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DLOADER_H
#define X3DLOADER_H

#include "X3DTypes.h"

namespace X3DTK {

class NodeCreationProxy;

namespace X3D {

class Scene;
class X3DFileValidator;

}

/// Abstract class defining an interface for an X3D file loader.

class X3DLoader
{
public:
  /// Constructor.
  X3DLoader();
  /// Destructor.
  virtual ~X3DLoader() = 0;
  
  /// Sets the componentVisitor.
  void setComponentCreator(X3DComponentCreator *component);
  /// Sets the fileValidator.
  void setFileValidator(X3D::X3DFileValidator *fileValidator);  
  
  /// Loads a scene from the file. If fileValidation is true, then the file will be 
  /// verified and messages will be written if the file is invalid. Nevertheless the 
  /// content of the scene is not deleted.
  X3D::Scene *load(const char *file, bool fileValidation = true) const;
  
  /// Gets the fileValidator.
  inline X3D::X3DFileValidator *getFileValidator() const {return fileValidator;};  
  
protected:
  /// NodeCreationProxy.
  NodeCreationProxy *ncp;
  /// FileValidator.
  X3D::X3DFileValidator *fileValidator;  
  
  /// Loads a scene from the file. This method has to be redefined.
  virtual X3D::Scene *loadFile(const char *file, bool fileValidation) const = 0;
};

}

#endif
