////////////////////////////////////////////////////////////////////////////////
//                            X3DMeshGroupingNode.h                           //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DMESHGROUPINGNODE
#define X3DMESHGROUPINGNODE

#include "X3DMeshNode.h"

namespace X3DTK {
namespace Mesh {

/// Base class defining a Grouping Node for the Mesh graph scene.

class X3DGroupingNode : public X3DNode
{
public:
  /// Constructor.
  X3DGroupingNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DGroupingNode();

  /// Adds a child to the Children attribute. Returns TRUE if succeeds.
  bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child. Return TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return _children;};
  
protected:
  /// Copy constructor.		  
  X3DGroupingNode(const X3DGroupingNode &N);

private:  
  /// Children attribute.
  MFNode _children;
};

}
}

#endif
