////////////////////////////////////////////////////////////////////////////////
//                            X3DShapeNode.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DSHAPENODE_H
#define X3DSHAPENODE_H

#include "X3DTypes.h"
#include "X3DChildNode.h"

namespace X3DTK {
namespace X3D {

class X3DGeometryNode;
class X3DAppearanceNode;

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">Shape 
/// component</a> defining an X3DShapeNode
/// (<a href="../../doc/X3DSpecs/Part01/components/Shape.html#X3DShapeNode">
/// documentation</a>).

class X3DShapeNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DShapeNode();
  /// Constructs an X3DShapeNode from its attributes.
  X3DShapeNode(const X3DGeometryNode *geometry, 
               const X3DAppearanceNode *appearance);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DShapeNode();
  
  /// Sets the Geometry attribute.
  void setGeometry(const X3DGeometryNode *geometry);
  /// Sets the Appearance attribute.
  void setAppearance(const X3DAppearanceNode *appearance);

  /// Gets the Geometry attribute.
  inline SFNode getGeometry() const {return _geometry;};
  /// Gets the Appearance attribute.
  inline SFNode getAppearance() const {return _appearance;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);
  
protected:
  /// Copy constructor.	       
  X3DShapeNode(const X3DShapeNode &N);

private:  
  SFNode _geometry;
  SFNode _appearance;  
};

}
}

#endif
