////////////////////////////////////////////////////////////////////////////////
//                            X3DTextureCoordinateNode.h                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DTEXTURECOORDINATENODE_H
#define X3DTEXTURECOORDINATENODE_H

#include "X3DTypes.h"
#include "X3DGeometricPropertyNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/texturing.html">
/// Texturing component</a> defining an X3DTextureCoordinateNode
/// (<a href="../../doc/X3DSpecs/Part01/components/texturing.html#X3DTextureCoordinateNode">
/// documentation</a>).

class X3DTextureCoordinateNode : public X3DGeometricPropertyNode
{
public:  
  /// Constructor.
  X3DTextureCoordinateNode();
  /// Constructs an X3DTextureCoordinateNode from its attributes.
  X3DTextureCoordinateNode(const MFVec2f &point);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DTextureCoordinateNode();

  /// Sets the Point attribute.
  void setPoint(const MFVec2f &point);
    
  /// Gets the Point attribute.
  inline const MFVec2f &getPoint() const {return _point;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.	
  X3DTextureCoordinateNode(const X3DTextureCoordinateNode &N);  

private:  
  /// The point array.
  MFVec2f _point;  
};

}
}

#endif
