#include "DefaultDFSShapeWalker.h"
#include "NodeWalkingProxy.h"
#include "NodeVisitingProxy.h"
#include "X3DShapeNode.h"
#include "Appearance.h"

namespace X3DTK {
namespace X3D {

DefaultDFSShapeWalker::DefaultDFSShapeWalker()
: DFSShapeWalker()
{
  defineNewWalkingFunction<DefaultDFSShapeWalker, X3DShapeNode>(&DefaultDFSShapeWalker::walkX3DShapeNode);  
  defineNewWalkingFunction<DefaultDFSShapeWalker, Appearance>(&DefaultDFSShapeWalker::walkAppearance);  
}

DefaultDFSShapeWalker::~DefaultDFSShapeWalker()
{
}

void DefaultDFSShapeWalker::walkX3DShapeNode(X3DShapeNode *N) const
{
  nodeVisitingProxy->enter(N);
  
  SFNode C;
  C = N->getAppearance();
  if ((C != 0) && (nodeVisitingProxy->walkOn(N, C)))
    nodeWalkingProxy->walk(C);
  
  C = N->getGeometry();
  if ((C != 0) && (nodeVisitingProxy->walkOn(N, C)))
    nodeWalkingProxy->walk(C);
    
  nodeVisitingProxy->leave(N);
}

void DefaultDFSShapeWalker::walkAppearance(Appearance *A) const
{
  nodeVisitingProxy->enter(A);
  
  SFNode C;
  C = A->getMaterial();
  if ((C != 0) && (nodeVisitingProxy->walkOn(A, C)))
    nodeWalkingProxy->walk(C);
    
  C = A->getTexture();
  if ((C != 0) && (nodeVisitingProxy->walkOn(A, C)))
    nodeWalkingProxy->walk(C);

  C = A->getTextureTransform();
  if ((C != 0) && (nodeVisitingProxy->walkOn(A, C)))
    nodeWalkingProxy->walk(C);

  C = A->getLineProperties();
  if ((C != 0) && (nodeVisitingProxy->walkOn(A, C)))
    nodeWalkingProxy->walk(C);

  C = A->getFillProperties();
  if ((C != 0) && (nodeVisitingProxy->walkOn(A, C)))
    nodeWalkingProxy->walk(C);
    
  nodeVisitingProxy->leave(A);
}

}
}
