#include "GLSphere.h"
#include "Sphere.h"
#include "SphereDrawArray.h"

namespace X3DTK {
namespace GL {

Sphere::Sphere()
: X3DGeometry3DNode(), _radius(0.0f), _sphereArray(0)
{
  defineTypeName("Sphere", "GLGeometry3D");
}

Sphere::Sphere(const Sphere &S)
: X3DGeometry3DNode(S), _radius(S._radius), _sphereArray(0)
{
}

SFNode Sphere::clone() const
{
  return new Sphere(*this);
}

Sphere::~Sphere()
{
  _sphereArray->removeInstance();
}

void Sphere::setRadius(SFFloat radius)
{
  _radius = radius;
}

void Sphere::setSphereDrawArray(SphereDrawArray *sphereArray)
{
  _sphereArray = sphereArray;
}

void Sphere::update()
{
  if (x3dReference == 0)
    return;
    
  X3D::Sphere *S = static_cast<X3D::Sphere *>(x3dReference);

  _radius = S->getRadius();
  _sphereArray = SphereDrawArray::getInstanceOfSection(64); 
}

void Sphere::render() const
{
  glEnable(GL_CULL_FACE);
  glFrontFace(GL_CW);
  glCullFace(GL_BACK);    
  glPushMatrix();
  glScalef(_radius, _radius, _radius);  

  glInterleavedArrays(GL_N3F_V3F, 0, _sphereArray->getSphereVertexArrayAddress());
  glDrawElements(GL_TRIANGLES, _sphereArray->getSphereSize(), GL_UNSIGNED_INT, _sphereArray->getSphereIndexArrayAddress());

  glPopMatrix();
  glDisable(GL_CULL_FACE);
}

}
}
