#include "GLImageTexture.h"
#include "ImageTexture.h"

#include <iostream>
#include <qimage.h>
#include <qgl.h>
#include <qstring.h>

#include "Image.h"

using namespace std;

namespace X3DTK {
namespace GL {

ImageTexture::ImageTexture()
: X3DTexture2DNode(), _texName(0), _textureTransform(16)
{
  defineTypeName("ImageTexture", "GLTexturing");
  
  _textureTransform[0] = 1.0f;
  _textureTransform[1] = 0.0f;
  _textureTransform[2] = 0.0f;
  _textureTransform[3] = 0.0f;
  
  _textureTransform[4] = 0.0f;
  _textureTransform[5] = 1.0f;
  _textureTransform[6] = 0.0f;
  _textureTransform[7] = 0.0f;

  _textureTransform[8] = 0.0f;
  _textureTransform[9] = 0.0f;
  _textureTransform[10] = 1.0f;
  _textureTransform[11] = 0.0f;

  _textureTransform[12] = 0.0f;
  _textureTransform[13] = 0.0f;
  _textureTransform[14] = 0.0f;
  _textureTransform[15] = 1.0f;
}

ImageTexture::ImageTexture(const ImageTexture &I)
: X3DTexture2DNode(I), _texName(I._texName), _textureTransform(I._textureTransform)
{
}

SFNode ImageTexture::clone() const
{
  return new ImageTexture(*this);
}

ImageTexture::~ImageTexture()
{
  glDeleteTextures(1, &_texName);
}

void ImageTexture::setTexName(GLuint texName)
{
  _texName = texName;
}

void ImageTexture::setTextureTransform(const MFFloat &textureTransform)
{
  _textureTransform = textureTransform;
}

void ImageTexture::update()
{
  if (x3dReference == 0)
    return;
    
  X3D::ImageTexture *I = static_cast<X3D::ImageTexture *>(x3dReference);
    
  MFString url = I->getUrl();
  if (url.empty())
    return;
  
  const char *file = SFString(url.front());
  
  QImage img(file);
  
  if (img.isNull())
  {
    cerr << "unable to load the texture from " << file << ", unsupported file format" << endl;
    return;
  }
  // 1E-3 needed. Just try with width=128 and see !
  int newWidth  = 1<<(int)(1+log(img.width() -1+1E-3) / log(2.0));
  int newHeight = 1<<(int)(1+log(img.height()-1+1E-3) / log(2.0));

  //set the ratios for the next geometry node
  SFFloat ratioS = (float)(img.width())/(float)(newWidth);
  SFFloat ratioT = (float)(img.height())/(float)(newHeight);
  
  _textureTransform[0] = ratioS;
  _textureTransform[12] = -ratioS + 2.0f;
  
  _textureTransform[5] = ratioT;
  _textureTransform[13] = 1.0f - ratioT;
        
  if ((img.width() != newWidth) || (img.height() != newHeight))
    img = img.copy(0, 0, newWidth, newHeight);

  QImage glImg = QGLWidget::convertToGLFormat(img);  // flipped 32bit RGBA

  // GL commands
  glGenTextures(1, &_texName);
  
  glBindTexture(GL_TEXTURE_2D, _texName);
  
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

  if (I->getRepeatS())
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  else
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    
  if (I->getRepeatT())
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  else
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    
  //glTexImage2D(GL_TEXTURE_2D, 0, 3, glImg.width(), glImg.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, glImg.bits());
  
  Image texImage(file);
    
  glTexImage2D(GL_TEXTURE_2D, 0, 3, texImage.getWidth(), texImage.getHeight(), 0, texImage.getPixelFormat(), GL_UNSIGNED_BYTE, texImage.getData());
}

void ImageTexture::render() const
{
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  glBindTexture(GL_TEXTURE_2D, _texName);
  
  glMatrixMode(GL_TEXTURE);
  glLoadMatrixf(&_textureTransform.front());
}

}
}
