/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            Box.h                                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef BOX_H
#define BOX_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining a Box 
/// (<a href="../../doc/X3DSpecs/Part01/components/geometry3D.html#Box">
/// documentation</a>).

class Box : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Box();
  /// Constructs a Box from \a size, the vector representing the diagonal of the Box. 
  /// The Box is centered on (0, 0, 0).
  Box(const SFVec3f &size);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~Box();
  
  /// Sets the size attribute.
  void setSize(const SFVec3f &size);
  
  /// Gets the size attribute.
  inline const SFVec3f &getSize() const {return _size;};
  
  /// Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.
  Box(const Box &B);

private:  
  /// Size attribute.
  SFVec3f _size;
};

}
}

#endif
