#include "IndexedFaceSet.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

IndexedFaceSet::IndexedFaceSet()
: X3DComposedGeometryNode(), _convex(true), _creaseAngle(0.0f)
{
  defineTypeName("IndexedFaceSet", "Geometry3D");
}

IndexedFaceSet::IndexedFaceSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, const MFInt32 &colorIndex, SFBool colorPerVertex, SFBool convex, const MFInt32 &coordIndex, SFFloat creaseAngle, const MFInt32 &normalIndex, SFBool normalPerVertex, SFBool solid, const MFInt32 &texCoordIndex) 
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
_colorIndex(colorIndex), _convex(convex), _coordIndex(coordIndex), _creaseAngle(creaseAngle), _normalIndex(normalIndex), _texCoordIndex(texCoordIndex)
{
  defineTypeName("IndexedFaceSet", "Geometry3D");
}
           
IndexedFaceSet::IndexedFaceSet(const IndexedFaceSet &I)
: X3DComposedGeometryNode(I), _colorIndex(I._colorIndex), _convex(I._convex), _coordIndex(I._coordIndex), _creaseAngle(I._creaseAngle), _normalIndex(I._normalIndex), _texCoordIndex(I._texCoordIndex)
{
}

SFNode IndexedFaceSet::clone() const
{
  return new IndexedFaceSet(*this);
}

IndexedFaceSet::~IndexedFaceSet()
{
}
  
void IndexedFaceSet::setColorIndex(const MFInt32 &colorIndex)
{
  _colorIndex = colorIndex;
  if ((getNormalPerVertex()) && ((_colorIndex.empty()) || (_colorIndex.back() != -1)))
    _colorIndex.push_back(-1);
}

void IndexedFaceSet::setConvex(SFBool convex)
{
  _convex = convex;
}

void IndexedFaceSet::setCoordIndex(const MFInt32 &coordIndex)
{
  _coordIndex = coordIndex;
  if ((_coordIndex.empty()) || (_coordIndex.back() != -1))
    _coordIndex.push_back(-1);
}

void IndexedFaceSet::setCreaseAngle(SFFloat creaseAngle)
{
  _creaseAngle = creaseAngle;
}

void IndexedFaceSet::setNormalIndex(const MFInt32 &normalIndex)
{
  _normalIndex = normalIndex;
  if ((getNormalPerVertex()) && ((_normalIndex.empty()) || (_normalIndex.back() != -1)))
    _normalIndex.push_back(-1);
}

void IndexedFaceSet::setTexCoordIndex(const MFInt32 &texCoordIndex)  
{
  _texCoordIndex = texCoordIndex;
  if ((_texCoordIndex.empty()) || (_texCoordIndex.back() != -1))
    _texCoordIndex.push_back(-1);
 
}

void IndexedFaceSet::load(const X3DFileElement *element)
{
  X3DComposedGeometryNode::load(element);
 
  int index;
  index = element->getIndexAttribute("colorIndex");
  if (index != -1)
    setColorIndex(MFInt32(element->getAttribute(index)));

  index = element->getIndexAttribute("convex");  
  if (index != -1)
    _convex = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("coordIndex");
  if (index != -1)
    setCoordIndex(MFInt32(element->getAttribute(index)));

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    _creaseAngle = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("normalIndex");
  if (index != -1)
    setNormalIndex(MFInt32(element->getAttribute(index)));
 
  index = element->getIndexAttribute("texCoordIndex");
  if (index != -1)
    setTexCoordIndex(MFInt32(element->getAttribute(index)));
}

SFString IndexedFaceSet::write() const
{
  SFString attr;
  attr = X3DComposedGeometryNode::write();
  
  if (!_colorIndex.empty()) 
    attr += " colorIndex=\"" + toSFString(_colorIndex) + "\"";
  if (!_convex)  
    attr += " convex=\"FALSE\"";
  if (!_coordIndex.empty())  
    attr += " coordIndex=\"" + toSFString(_coordIndex) + "\"";
  if (_creaseAngle != 0.0f)
    attr += " creaseAngle=\"" + toSFString(_creaseAngle) + "\"";
  if (!_normalIndex.empty())  
    attr += " normalIndex=\"" + toSFString(_normalIndex) + "\"";
  if (!_texCoordIndex.empty())  
    attr += " texCoordIndex=\"" + toSFString(_texCoordIndex) + "\"";
  
  return attr;
}

}
}
