/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            IndexedFaceSet.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef INDEXEDFACESET_H
#define INDEXEDFACESET_H

#include "X3DTypes.h"
#include "X3DComposedGeometryNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining an IndexedFaceSet
/// (<a href="../../doc/X3DSpecs/Part01/components/geometry3D.html#IndexedFaceSet">
/// documentation</a>).

class IndexedFaceSet : public X3DComposedGeometryNode
{
public:
  /// Constructor.
  IndexedFaceSet();
  /// Constructs an IndexedFaceSet node from its attributes.
  IndexedFaceSet(const X3DColorNode *color,
                 const X3DCoordinateNode *coord,
                 const X3DNormalNode *normal,
                 const X3DTextureCoordinateNode *texCoord,
                 SFBool ccw,
                 const MFInt32 &colorIndex,
                 SFBool colorPerVertex,
                 SFBool convex,
                 const MFInt32 &coordIndex,
                 SFFloat creaseAngle,
                 const MFInt32 &normalIndex,                 
                 SFBool normalPerVertex,
                 SFBool solid,
                 const MFInt32 &texCoordIndex);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~IndexedFaceSet();
  
  /// Sets the ColorIndex attribute.
  void setColorIndex(const MFInt32 &colorIndex);
  /// Sets the Convex attribute.
  void setConvex(SFBool convex);
  /// Sets the CoordIndex attribute.
  void setCoordIndex(const MFInt32 &coordIndex);
  /// Sets the CreaseAngle attribute.
  void setCreaseAngle(SFFloat creaseAngle);
  /// Sets the NormalIndex attribute.
  void setNormalIndex(const MFInt32 &normalIndex);
  /// Sets the TexCoordIndex attribute.
  void setTexCoordIndex(const MFInt32 &texCoordIndex);
    
  /// Gets the ColorIndex attribute.
  inline const MFInt32 &getColorIndex() const {return _colorIndex;};
  /// Gets the Convex attribute.
  inline SFBool getConvex() const {return _convex;};  
  /// Gets the CoordIndex attribute.
  inline const MFInt32 &getCoordIndex() const {return _coordIndex;};
  /// Gets the CreaseAngle attribute.
  inline SFFloat getCreaseAngle() const {return _creaseAngle;};
  /// Gets the NormalIndex attribute.
  inline const MFInt32 &getNormalIndex() const {return _normalIndex;};
  /// Gets the TexCoordIndex attribute.
  inline const MFInt32 &getTexCoordIndex() const {return _texCoordIndex;};
  
  /// Loads the attributes from a X3DFileElement e.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.       
  IndexedFaceSet(const IndexedFaceSet &G);

private:  
  /// Color index attribute.
  MFInt32 _colorIndex;
  /// Convex attribute.
  SFBool _convex;
  /// Coord index attribute.
  MFInt32 _coordIndex;
  /// Crease angle attribute.
  SFFloat _creaseAngle; 
  /// Normal index attribute.
  MFInt32 _normalIndex;
  /// texCoord index attribute.
  MFInt32 _texCoordIndex;
};

}
}

#endif
