#include "Transform.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

Transform::Transform()
: X3DGroupingNode(), X3DBoundedObject(), _center(SFVec3f(0.0f, 0.0f, 0.0f)), _rotation(SFRotation(0.0f, 0.0f, 1.0f, 0.0f)), _scale(SFVec3f(1.0f, 1.0f, 1.0f)), _scaleOrientation(SFRotation(0.0f, 0.0f, 1.0f, 0.0f)), _translation(SFVec3f(0.0f, 0.0f, 0.0f))
{
  defineTypeName("Transform", "Grouping");
}

Transform::Transform(const SFVec3f &center, const SFRotation &rotation, const SFVec3f &scale,  const SFRotation &scaleOrientation,  const SFVec3f &translation,  const SFVec3f &bboxCenter,  const SFVec3f &bboxSize)
: X3DGroupingNode(), X3DBoundedObject(bboxCenter, bboxSize), _center(center), _rotation(rotation), _scale(scale), _scaleOrientation(scaleOrientation), _translation(translation)
{
  defineTypeName("Transform", "Grouping");
}

Transform::Transform(const Transform &T)
: X3DGroupingNode(T), X3DBoundedObject(T), _center(T._center), _rotation(T._rotation), _scale(T._scale), _scaleOrientation(T._scaleOrientation), _translation(T._translation)
{
}

SFNode Transform::clone() const
{
  return new Transform(*this);
}

Transform::~Transform()
{
}

void Transform::setCenter(const SFVec3f &center)
{
  _center = center;
}

void Transform::setRotation(const SFRotation &rotation)
{
  _rotation = rotation;
}

void Transform::setScale(const SFVec3f &scale)
{
  _scale = scale;
}

void Transform::setScaleOrientation(const SFRotation &scaleOrientation)
{
  _scaleOrientation = scaleOrientation;
}

void Transform::setTranslation(const SFVec3f &translation)
{
  _translation = translation;
}

void Transform::load(const X3DFileElement *element)
{
  X3DBoundedObject::load(element);
  
  int index;  
  index = element->getIndexAttribute("center");
  if (index != -1)
    _center = SFVec3f(element->getAttribute(index)); 
  
  index = element->getIndexAttribute("rotation"); 
  if (index != -1)
    _rotation = SFRotation(element->getAttribute(index)); 
  
  index = element->getIndexAttribute("scale");
  if (index != -1)
    _scale = SFVec3f(element->getAttribute(index)); 
 
  index = element->getIndexAttribute("scaleOrientation"); 
  if (index != -1)
    _scaleOrientation = SFRotation(element->getAttribute(index)); 
   
  index = element->getIndexAttribute("translation"); 
  if (index != -1)
    _translation = SFVec3f(element->getAttribute(index)); 
}

SFString Transform::write() const
{
  SFString attr;
  if (_center != SFVec3f(0.0f, 0.0f, 0.0f))
    attr += " center=\"" + toSFString(_center) + "\"";
  if (_rotation != SFRotation(0.0f, 0.0f, 1.0f, 0.0f))
    attr += " rotation=\"" + toSFString(_rotation) + "\"";
  if (_scale != SFVec3f(1.0f, 1.0f, 1.0f))
    attr += " scale=\"" + toSFString(_scale) + "\"";
  if (_scaleOrientation != SFRotation(0.0f, 0.0f, 1.0f, 0.0f))  
    attr += " scaleOrientation=\"" + toSFString(_scaleOrientation) + "\"";
  if (_translation != SFVec3f(0.0f, 0.0f, 0.0f))
    attr += " translation=\"" + toSFString(_translation) + "\"";
 
  return attr;   
}

}
}
