/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DGroupingNode.h                               //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DGROUPINGNODE
#define X3DGROUPINGNODE

#include "X3DTypes.h"
#include "X3DChildNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/group.html">
/// Grouping component</a> defining an X3DGroupingNode
/// (<a href="../../doc/X3DSpecs/Part01/components/grouping.html#X3DGroupingNode">
/// documentation</a>).

class X3DGroupingNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DGroupingNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DGroupingNode();

  /// Adds a child to the Children attribute. Returns TRUE if succeeds.
  bool addChild(const SFAbstractNode &N);
  /// Sets the child. Behaves like addChild.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child. Return TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return _children;};
  
protected:
  /// Copy constructor.		  
  X3DGroupingNode(const X3DGroupingNode &N);

private:  
  /// Children attribute.
  MFNode _children;
};

}
}

#endif
