/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            IndexedLineSet.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef INDEXEDLINESET_H
#define INDEXEDLINESET_H

#include "X3DTypes.h"
#include "X3DGeometryNode.h"

namespace X3DTK {
namespace X3D {

class X3DColorNode;
class X3DCoordinateNode;

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/rendering.html">
/// Rendering component</a> defining an IndexedLineSet
/// (<a href="../../doc/X3DSpecs/Part01/components/rendering.html#IndexedLineSet">
/// documentation</a>).

class IndexedLineSet : public X3DGeometryNode
{
public:
  /// Constructor.
  IndexedLineSet();
  /// Constructs an IndexedLineSet from its attributes
  IndexedLineSet(const X3DColorNode *color,
                 const X3DCoordinateNode *coord,
                 const MFInt32 &colorIndex,
                 SFBool colorPerVertex,
                 const MFInt32 &coordIndex,
                 SFFloat lineWidth);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~IndexedLineSet();
  
  /// Sets the Color attribute.
  void setColor(const X3DColorNode *color);
  /// Sets the Coord attribute.
  void setCoord(const X3DCoordinateNode *coord);
  /// Sets the ColorIndex attribute.
  void setColorIndex(const MFInt32 &colorIndex);
  /// Sets the ColorPerVertex attribute.
  void setColorPerVertex(SFBool colorPerVertex);
  /// Sets the CoordIndex attribute.
  void setCoordIndex(const MFInt32 &coordIndex);
  /// Sets the LineWidth attribute.
  void setLineWidth(SFFloat lineWidth);
  
  /// Gets the Color attribute.
  inline SFNode getColor() const {return _color;};
  /// Gets the Coord attribute.
  inline SFNode getCoord() const {return _coord;};
  /// Gets the ColorIndex attribute.
  inline const MFInt32 &getColorIndex() const {return _colorIndex;};
  /// Gets the ColorPerVertex attribute.
  inline SFBool getColorPerVertex() const {return _colorPerVertex;};
  /// Gets the CoordIndex attribute.
  inline const MFInt32 &getCoordIndex() const {return _coordIndex;};
  /// Gets the LineWidth attribute.
  inline SFFloat getLineWidth() const {return _lineWidth;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFAbstractNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFAbstractNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFAbstractNode &N);
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor.      
  IndexedLineSet(const IndexedLineSet &I);

private:  
  /// Color node.
  SFNode _color;
  /// Coord node.
  SFNode _coord;
  /// Color index attribute.
  MFInt32 _colorIndex;
  /// Color per vertex attribute.
  SFBool _colorPerVertex;
  /// Coord index attribute.
  MFInt32 _coordIndex;
  /// Line width attribute.
  SFFloat _lineWidth;  
};

}
}

#endif
