/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            FillProperties.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef FILLPROPERTIES_H
#define FILLPROPERTIES_H

#include "X3DTypes.h"
#include "X3DAppearanceChildNode.h"

namespace X3DTK {
namespace X3D {

/// Class of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">Shape 
/// component</a> defining a FillProperties
/// (<a href="../../doc/X3DSpecs/Part01/components/Shape.html#FillProperties">
/// documentation</a>).

class FillProperties : public X3DAppearanceChildNode
{
public:
  /// Constructor.
  FillProperties();
  /// Constructs a Material from its attributes.
  FillProperties(SFString fillStyle, 
                 SFInt32 hatchStyle, 
		 SFColor hatchColor);
  /// Clones the node.
  virtual SFNode clone() const;		 
  
  /// Destructor.
  virtual ~FillProperties();

  /// Sets the FillStyle attribute.
  void setFillStyle(SFString fillStyle);
  /// Sets the HatchStyle attribute.
  void setHatchStyle(SFInt32 hatchStyle);
  /// Sets the HatchColor attribute.
  void setHatchColor(SFColor hatchColor);
  
  /// Sets the FillStyle attribute.
  inline SFString getFillStyle() const {return _fillStyle;};
  /// Sets the HatchStyle attribute.
  inline SFInt32 getHatchStyle() const {return _hatchStyle;};
  /// Sets the HatchColor attribute.
  inline SFColor getHatchColor() const {return _hatchColor;};

  /// Loads the attributes from a X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;

protected:
  /// Copy constructor. 
  FillProperties(const FillProperties &F);

private:
  /// Fill style.
  SFString _fillStyle;
  /// Hatch style.
  SFInt32 _hatchStyle;
  /// Hatch color.
  SFColor _hatchColor;
};

}
}

#endif
