#include "LineProperties.h"

namespace X3DTK {
namespace X3D {

LineProperties::LineProperties()
: X3DAppearanceChildNode(), _lineStyle(0), _width(0.0f)
{
  defineTypeName("LineProperties", "Shape");
}

LineProperties::LineProperties(SFInt32 lineStyle, SFFloat width)
: X3DAppearanceChildNode(), _lineStyle(lineStyle), _width(width)
{
  defineTypeName("LineProperties", "Shape");
}

LineProperties::LineProperties(const LineProperties &L)
: X3DAppearanceChildNode(L), _lineStyle(L._lineStyle), _width(L._width)
{
}

SFNode LineProperties::clone() const
{
  return new LineProperties(*this);
}

LineProperties::~LineProperties()
{
}

void LineProperties::setLineStyle(SFInt32 lineStyle)
{
  _lineStyle = lineStyle;
}

void LineProperties::setWidth(SFFloat width)
{
  _width = width;
}

void LineProperties::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("lineStyle");
  if (index != -1)
    _lineStyle = element->getAttribute(index).toInt();
   
  index = element->getIndexAttribute("width");
  if (index != -1)
    _width = element->getAttribute(index).toFloat(); 
}

SFString LineProperties::write() const
{
  SFString attr;
  if (_lineStyle != 0)
    attr += " lineStyle = \"" + toSFString(_lineStyle) + "\"";  
  if (_width != 0.0f)
    attr += " width = \"" + toSFString(_width) + "\"";
      
  return attr;
}

}
}
