#include "Material.h"

namespace X3DTK {
namespace X3D {

Material::Material()
: X3DMaterialNode(), _ambientIntensity(0.2f), _diffuseColor(SFColor(0.8f, 0.8f, 0.8f)), _emissiveColor(SFColor(0.0f, 0.0f, 0.0f)), _shininess(0.2f), _specularColor(SFColor(0.0f, 0.0f, 0.0f)), _transparency(0.0f)
{ 
  defineTypeName("Material", "Shape");
}

Material::Material(SFFloat ambientIntensity, SFColor diffuseColor, SFColor emissiveColor, SFFloat shininess, SFColor specularColor, SFFloat transparency)
: X3DMaterialNode(), _ambientIntensity(ambientIntensity), _diffuseColor(diffuseColor), _emissiveColor(emissiveColor), _shininess(shininess), _specularColor(specularColor), _transparency(transparency)
{
  defineTypeName("Material", "Shape");
}

Material::Material(const Material &M)
: X3DMaterialNode(M), _ambientIntensity(M._ambientIntensity), _diffuseColor(M._diffuseColor), _emissiveColor(M._emissiveColor), _shininess(M._shininess), _specularColor(M._specularColor), _transparency(M._transparency)
{
}

SFNode Material::clone() const
{
  return new Material(*this);
}

Material::~Material()
{
}

void Material::setAmbientIntensity(SFFloat ambientIntensity)
{
  _ambientIntensity = ambientIntensity;
}

void Material::setDiffuseColor(SFColor diffuseColor)
{
  _diffuseColor = diffuseColor;
}

void Material::setEmissiveColor(SFColor emissiveColor)  
{
  _emissiveColor = emissiveColor;
}

void Material::setShininess(SFFloat shininess)  
{
  _shininess = shininess;
}

void Material::setSpecularColor(SFColor specularColor)  
{
  _specularColor = specularColor;  
}

void Material::setTransparency(SFFloat transparency)  
{
  _transparency = transparency;
}

void Material::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("ambientIntensity");
  if (index != -1)
    _ambientIntensity = element->getAttribute(index).toFloat();

  index = element->getIndexAttribute("diffuseColor");
  if (index != -1)
    _diffuseColor = SFColor(element->getAttribute(index));

  index = element->getIndexAttribute("emissiveColor");
  if (index != -1)
    _emissiveColor = SFColor(element->getAttribute(index));

  index = element->getIndexAttribute("shininess");
  if (index != -1)
    _shininess = element->getAttribute(index).toFloat();

  index = element->getIndexAttribute("specularColor");
  if (index != -1)
    _specularColor = SFColor(element->getAttribute(index));
}

SFString Material::write() const
{
  SFString attr;
  if (_ambientIntensity != 0.2f)
    attr += " ambientIntensity=\"" + toSFString(_ambientIntensity) + "\"";
  if (_diffuseColor != SFColor(0.8f, 0.8f, 0.8f))  
    attr += " diffuseColor=\"" + toSFString(_diffuseColor) + "\"";
  if (_emissiveColor != SFColor(0.0f, 0.0f, 0.0f))  
    attr += " emissiveColor=\"" + toSFString(_emissiveColor) + "\"";
  if (_shininess != 0.2f)
    attr += " shininess=\"" + toSFString(_shininess) + "\"";
  if (_specularColor != SFColor(0.0f, 0.0f, 0.0f))
    attr += " specularColor=\"" + toSFString(_specularColor) + "\"";
  if (_transparency != 0.0f)
    attr += " transparency=\"" + toSFString(_transparency) + "\"";
      
  return attr;
}

}
}
