/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            X3DTexture2DNode.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DTEXTURE2DNODE_H
#define X3DTEXTURE2DNODE_H

#include "X3DTextureNode.h"

namespace X3DTK {
namespace X3D {

/// Base class of the <a href="../../doc/X3DSpecs/Part01/components/texturing.html">
/// Texturing component</a> defining an X3DTextureNode
/// (<a href="../../doc/X3DSpecs/Part01/components/texturing.html#X3DTextureNode">
/// documentation</a>).

class X3DTexture2DNode : public X3DTextureNode
{ 
public:  
  /// Constructor.
  X3DTexture2DNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DTexture2DNode();
  
  /// Sets repeatS attribute.
  void setRepeatS(SFBool repeatS);
  /// Sets repeatT attribute.
  void setRepeatT(SFBool repeatT);
 
  /// Gets repeatS attribute.
  inline SFBool getRepeatS() {return _repeatS;};
  /// Gets repeatT attribute.
  inline SFBool getRepeatT() {return _repeatT;};
  
  /// Loads the attributes from X3DFileElement *.
  virtual void load(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString write() const;
  
protected:
  /// Copy constructor.	
  X3DTexture2DNode(const X3DTexture2DNode &N);

private:
  SFBool _repeatS;
  SFBool _repeatT;  
};

}
}

#endif
