/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            Bbox.h                                         //
////////////////////////////////////////////////////////////////////////////////

#ifndef BBOX_H
#define BBOX_H

#include "X3DTypes.h"

namespace X3DTK {

/// Class providing a Bounding Box. The attributes are  a center and size a vector 
/// representing the diagonal.

class Bbox
{
public:
  /// Default constructor.
  Bbox();
  /// Constructor with center of the Bbox and its size.
  Bbox(const SFVec3f &center, const SFVec3f &size);
  /// Constructs th Bounding Box of the vertices of the node coord.
  explicit Bbox(const MFVec3f &coord);
  /// Copy constructor.
  Bbox::Bbox(const Bbox &B);
  
  /// Sets the center of the Bounding Box.
  inline void setCenter(const SFVec3f &center) {center_ = center;};
  /// Sets the Size of the Bounding Box.
  inline void setSize(const SFVec3f &size) {size_ = size;};
  
  /// Gets the center of the Bounding Box.
  inline SFVec3f getCenter() const {return center_;};
  /// Gets the size of the Bounding Box.
  inline SFVec3f getSize() const {return size_;};
  
private:
  SFVec3f center_;
  SFVec3f size_;  
};

}

#endif
