#include "BboxUpdaterGlobalVariables.h"

using namespace std;

namespace X3DTK {
namespace X3D {

BboxUpdaterGlobalVariables::BboxUpdaterGlobalVariables()
: GlobalVariables(), _staticProcessing(true)
{
}

BboxUpdaterGlobalVariables::~BboxUpdaterGlobalVariables()
{
  //delete the Bbox
  for (list<pair<SFNode, Bbox *> >::iterator it = _BBList.begin(); it != _BBList.end(); ++it)
    delete (*it).second;
}

void BboxUpdaterGlobalVariables::finish()
{
  for (list<pair<SFNode, Bbox *> >::iterator it = _BBList.begin(); it != _BBList.end(); ++it)
    delete (*it).second;
    
  _BBList.clear();
  _BBToMergeList.clear();
}

void BboxUpdaterGlobalVariables::setStaticProcessing(bool value)
{
  _staticProcessing = value;
}

void BboxUpdaterGlobalVariables::addBbox(SFNode N, Bbox *BB)
{
  _BBList.push_front(pair<SFNode, Bbox *>(N, BB));
}

void BboxUpdaterGlobalVariables::setShapeBbox(const Bbox &BB)
{
  _shapeBbox = BB;
}

void BboxUpdaterGlobalVariables::addBboxToMergeList(const Bbox &BB)
{
  _BBToMergeList.push_back(BB);
}

Bbox *BboxUpdaterGlobalVariables::getBbox(SFNode N) const
{
  for (list<pair<SFNode, Bbox *> >::const_iterator it = _BBList.begin(); it != _BBList.end(); ++it)
    if ((*it).first == N)
      return (*it).second;  
      
  return 0;   
}

Bbox BboxUpdaterGlobalVariables::mergeBbox()
{
  if (_BBToMergeList.empty())
    return Bbox(SFVec3f(0.0f, 0.0f, 0.0f), SFVec3f(-1.0f, -1.0f, -1.0f));
  
  Bbox BB = _BBToMergeList.front();
  SFVec3f min = BB.getCenter() - 0.5f*BB.getSize();
  SFVec3f max = BB.getCenter() + 0.5f*BB.getSize();

  //iterate mergingBboxStack
  for (list<Bbox>::const_iterator it = ++_BBToMergeList.begin(); it != _BBToMergeList.end(); ++it)
  {
    SFVec3f min2 = (*it).getCenter() - 0.5f*(*it).getSize();
    SFVec3f max2 = (*it).getCenter() + 0.5f*(*it).getSize();
    
    if (min2.x < min.x)
      min.x = min2.x;
    if (min2.y < min.y)
      min.y = min2.y;
    if (min2.z < min.z)
      min.z = min2.z;
  
    if (max2.x > max.x)
      max.x = max2.x;
    if (max2.y > max.y)
      max.y = max2.y;
    if (max2.z > max.z)
      max.z = max2.z;
  }
  
  //computing the new center
  SFVec3f center = 0.5f*(min + max);
  BB = Bbox(center, 2.0f*(max - center));
  
  //emptying the list.
  _BBToMergeList.clear();
  
  return BB;
}

}
}
