#include "BboxUpdaterGroupingVisitor.h"
#include "StaticGroup.h"
#include "X3DGroupingNode.h"
#include "Group.h"
#include "Transform.h"
#include "Bbox.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

BboxUpdaterGroupingVisitor::BboxUpdaterGroupingVisitor()
: GroupingVisitor()
{
  // WalkOn functions.
  defineNewWalkOnFunction<BboxUpdaterGroupingVisitor, StaticGroup>(&BboxUpdaterGroupingVisitor::walkOnStaticGroup);
  defineNewWalkOnFunction<BboxUpdaterGroupingVisitor, X3DGroupingNode>(&BboxUpdaterGroupingVisitor::walkOnX3DGroupingNode);

  // Leave functions.
  defineNewLeaveFunction<BboxUpdaterGroupingVisitor, StaticGroup>(&BboxUpdaterGroupingVisitor::leaveStaticGroup);
  defineNewLeaveFunction<BboxUpdaterGroupingVisitor, Group>(&BboxUpdaterGroupingVisitor::leaveGroup);
  defineNewLeaveFunction<BboxUpdaterGroupingVisitor, Transform>(&BboxUpdaterGroupingVisitor::leaveTransform);
  
  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<BboxUpdaterGlobalVariables>();
}

BboxUpdaterGroupingVisitor::~BboxUpdaterGroupingVisitor()
{
}

bool BboxUpdaterGroupingVisitor::walkOnStaticGroup(StaticGroup *S, SFAbstractNode Child) const
{
  return true;
}

bool BboxUpdaterGroupingVisitor::walkOnX3DGroupingNode(X3DGroupingNode *N, SFAbstractNode Child) const
{
  return true;
}

void BboxUpdaterGroupingVisitor::leaveStaticGroup(StaticGroup *S) const
{
  Bbox *BB = globalVariables->getBbox(S);
  if (BB == 0)
  {
    if (!globalVariables->getStaticProcessing())
    {
      BB = new Bbox(S->getBboxCenter(), S->getBboxSize());
      globalVariables->addBbox(S, BB);
    }
    else
    {  
      
      MFNode children = S->getChildren();
      for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
      {
        X3DBoundedObject *BO = dynamic_cast<X3DBoundedObject *>(*it);
        if (BO != 0)
          globalVariables->addBboxToMergeList(Bbox(BO->getBboxCenter(), BO->getBboxSize()));
      }
      
      BB = new Bbox(globalVariables->mergeBbox()); 
      globalVariables->addBbox(S, BB);
    
      //updating the Bbox attributes.
      S->setBboxCenter(BB->getCenter());
      S->setBboxSize(BB->getSize());
    }
  }
}

void BboxUpdaterGroupingVisitor::leaveGroup(Group *G) const
{
  Bbox *BB = globalVariables->getBbox(G);
  if (BB == 0)
  {
    MFNode children = G->getChildren();
    for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    {
      X3DBoundedObject *BO = dynamic_cast<X3DBoundedObject *>(*it);
      if (BO != 0)
        globalVariables->addBboxToMergeList(Bbox(BO->getBboxCenter(), BO->getBboxSize()));
    }
    
    BB = new Bbox(globalVariables->mergeBbox()); 
    globalVariables->addBbox(G, BB);
    
    //updating the Bbox attributes.
    G->setBboxCenter(BB->getCenter());
    G->setBboxSize(BB->getSize());
  }
}

void BboxUpdaterGroupingVisitor::leaveTransform(Transform *T) const
{
  Bbox *BB = globalVariables->getBbox(T);
  if (BB == 0)
  {
    MFNode children = T->getChildren();
    for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
    {
      X3DBoundedObject *BO = dynamic_cast<X3DBoundedObject *>(*it);
      if (BO != 0)
        globalVariables->addBboxToMergeList(Bbox(BO->getBboxCenter(), BO->getBboxSize()));
    }
    
    BB = new Bbox(globalVariables->mergeBbox()); 
    
    //change the coordinates of the Bbox in the coordinates of the parent node
    SFMatrix34f Tr = translation(T->getTranslation());
    SFMatrix34f C = translation(T->getCenter());
    SFVec3f v(T->getRotation().x, T->getRotation().y, T->getRotation().z);
    SFMatrix34f R = rotation(T->getRotation().angle, v);
    SFVec3f vs(T->getScaleOrientation().x, T->getScaleOrientation().y, T->getScaleOrientation().z);
    SFMatrix34f SR = rotation(T->getScaleOrientation().angle, vs);
    SFMatrix34f S = scale34(T->getScale().x, T->getScale().y, T->getScale().z);
    SFMatrix34f iSR = rotation(-T->getScaleOrientation().angle, vs);
    SFMatrix34f iC = translation(-1.0f*T->getCenter());
  
    SFMatrix34f transform = Tr*C*R*SR*S*iSR*iC;
  
    //comparing the 6 transformed points of the Bounding Box
    SFPoint3f Corner[8];
    Corner[0] = transform*SFPoint3f(BB->getCenter().x - 0.5f*BB->getSize().x, BB->getCenter().y - 0.5f*BB->getSize().y, BB->getCenter().z - 0.5f*BB->getSize().z);
    Corner[1] = transform*SFPoint3f(BB->getCenter().x - 0.5f*BB->getSize().x, BB->getCenter().y - 0.5f*BB->getSize().y, BB->getCenter().z + 0.5f*BB->getSize().z);
    Corner[2] = transform*SFPoint3f(BB->getCenter().x - 0.5f*BB->getSize().x, BB->getCenter().y + 0.5f*BB->getSize().y, BB->getCenter().z - 0.5f*BB->getSize().z);
    Corner[3] = transform*SFPoint3f(BB->getCenter().x - 0.5f*BB->getSize().x, BB->getCenter().y + 0.5f*BB->getSize().y, BB->getCenter().z + 0.5f*BB->getSize().z);
    Corner[4] = transform*SFPoint3f(BB->getCenter().x + 0.5f*BB->getSize().x, BB->getCenter().y - 0.5f*BB->getSize().y, BB->getCenter().z - 0.5f*BB->getSize().z);
    Corner[5] = transform*SFPoint3f(BB->getCenter().x + 0.5f*BB->getSize().x, BB->getCenter().y - 0.5f*BB->getSize().y, BB->getCenter().z + 0.5f*BB->getSize().z);
    Corner[6] = transform*SFPoint3f(BB->getCenter().x + 0.5f*BB->getSize().x, BB->getCenter().y + 0.5f*BB->getSize().y, BB->getCenter().z - 0.5f*BB->getSize().z);
    Corner[7] = transform*SFPoint3f(BB->getCenter().x + 0.5f*BB->getSize().x, BB->getCenter().y + 0.5f*BB->getSize().y, BB->getCenter().z + 0.5f*BB->getSize().z);
  
    SFPoint3f min = Corner[0];
    SFPoint3f max = Corner[0];
  
    for (unsigned int i = 0; i < 8; ++i)
    {
      if (Corner[i].x < min.x)
        min.x = Corner[i].x;
      if (Corner[i].y < min.y)
        min.y = Corner[i].y;
      if (Corner[i].z < min.z)
        min.z = Corner[i].z;
  
      if (Corner[i].x > max.x)
        max.x = Corner[i].x;
      if (Corner[i].y > max.y)
        max.y = Corner[i].y;
      if (Corner[i].z > max.z)
        max.z = Corner[i].z; 
    }    
   
    SFVec3f mid = max - min; 
    SFPoint3f center = min + 0.5f*mid;
    SFVec3f size = 2.0f*(max - center); 

    BB->setCenter(SFVec3f(center));
    BB->setSize(size);
    
    globalVariables->addBbox(T, BB);
    
    //updating the Bbox attributes.
    T ->setBboxCenter(BB->getCenter());
    T->setBboxSize(BB->getSize());
  }
}

}
}
