#include "MeshBuilder.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace Mesh {

VertexFData::VertexFData()
: _normal(SFVec3f(0.0f, 0.0f, 0.0f)), _color(SFColorRGBA(0.0f, 0.0f, 0.0f, 0.0f)), _texCoord(SFVec2f(0.0f, 0.0f))
{
}

void VertexFData::setNormal(const SFVec3f &normal)
{
  _normal = normal;
}

void VertexFData::setColor(const SFColorRGBA &color)
{
  _color = color;
}

void VertexFData::setTexCoord(const SFVec2f &texCoord)
{
  _texCoord = texCoord;
}

VertexData::VertexData()
: _point(SFVec3f(0.0f, 0.0f, 0.0f))
{
}

VertexData::~VertexData()
{
  for (std::list<std::pair<void *, VertexFData *> >::iterator it = _FDataList.begin(); it != _FDataList.end(); ++it)
    delete (*it).second;
}

void VertexData::setPoint(const SFVec3f &point)
{
  _point = point;
}

void VertexData::addFaceData(void *face, VertexFData *vdata)
{
  _FDataList.push_back(pair<void *, VertexFData *>(face, vdata));
}

VertexFData *VertexData::getFaceData(void *face) const 
{
  for (std::list<std::pair<void *, VertexFData *> >::const_iterator it = _FDataList.begin(); it != _FDataList.end(); ++it)
  {
    if ((*it).first == face)
      return (*it).second;
  }
  return 0;
}

EdgeData::EdgeData()
{
}

FaceData::FaceData()
{
}

void FaceData::setNormal(const SFVec3f &normal)
{
  _normal = normal;
}

void FaceData::setColor(const SFColorRGBA &color)
{
  _color = color;
}

MeshData::MeshData()
{
}

void MeshData::setVertexNormal(bool vertexNormal)
{
  _vertexNormal = vertexNormal;
}

void MeshData::setVertexColor(bool vertexColor)
{
  _vertexColor = vertexColor;
}

void MeshData::setVertexTexCoord(bool vertexTexCoord)
{
  _vertexTexCoord = vertexTexCoord;
}

void MeshData::setFaceNormal(bool faceNormal)
{
  _faceNormal = faceNormal;
}

void MeshData::setFaceColor(bool faceColor)
{
  _faceColor = faceColor;
}

/// Specialization of the template method start to initialize the building of the 
/// mesh.
template<>
void start<X3D::IndexedFaceSet, Mesh>(X3D::IndexedFaceSet *I, Mesh *mesh)
{
  MeshData md;  
  // normals
  md.setVertexNormal((I->getNormal() != 0) && I->getNormalPerVertex());
  md.setFaceNormal((I->getNormal() != 0) && !I->getNormalPerVertex());
  
  // colors
  md.setVertexColor((I->getColor() != 0) && I->getColorPerVertex());
  md.setFaceColor((I->getColor() != 0) && !I->getColorPerVertex());
  
  // texCoords
  md.setVertexTexCoord(I->getTexCoord() != 0);  
  
  mesh->data() = md;    
};

/// Specialization of the template method end to initialize the building of the 
/// mesh.
template<>
void end<X3D::IndexedFaceSet, Mesh>(X3D::IndexedFaceSet *I, Mesh *mesh)
{
  MFVec3f pointArray;
  MFVec3f normalArray;
  MFColorRGBA colorArray;
  MFVec2f texCoordArray;
  MFInt32 coordIndex;
  MFInt32 normalIndex;
  MFInt32 colorIndex;
  MFInt32 texCoordIndex;

  X3D::Coordinate *CoordinateNode = dynamic_cast<X3D::Coordinate *>(I->getCoord());
  if (CoordinateNode != 0)
    pointArray = CoordinateNode->getPoint();

  X3D::Normal *NormalNode = dynamic_cast<X3D::Normal *>(I->getNormal());
  if (NormalNode != 0)
    normalArray = NormalNode->getVector();
  
  if (I->getColor() != 0)
  {
    if (I->getColor()->getTypeName() == "ColorRGBA")
      colorArray = dynamic_cast<X3D::ColorRGBA *>(I->getColor())->getColor();
    else  
      colorArray = dynamic_cast<X3D::Color *>(I->getColor())->getColor();
  }
  
  X3D::TextureCoordinate *TexCoordNode = dynamic_cast<X3D::TextureCoordinate *>(I->getTexCoord());
  if (TexCoordNode != 0)
    texCoordArray = TexCoordNode->getPoint();
  
  coordIndex = I->getCoordIndex();
  colorIndex = I->getColorIndex();
  normalIndex = I->getNormalIndex();
  texCoordIndex = I->getTexCoordIndex();
      
  SFBool normalPerVertex = I->getNormalPerVertex();
  SFBool colorPerVertex = I->getColorPerVertex();
  
  MFInt32::const_iterator itCoordIndex = coordIndex.begin();
  MFInt32::const_iterator itColorIndex;
  MFInt32::const_iterator itNormalIndex;
  MFInt32::const_iterator itTexCoordIndex;
  
  if (colorIndex.empty())
    itColorIndex = coordIndex.begin();
  else
    itColorIndex = colorIndex.begin();  
  
  if (normalIndex.empty())
    itNormalIndex = coordIndex.begin();
  else
    itNormalIndex = normalIndex.begin();  
  
  if (texCoordIndex.empty())
    itTexCoordIndex = coordIndex.begin();
  else
    itTexCoordIndex = texCoordIndex.begin();
    
  Mesh::MFace::const_iterator itFace = mesh->getFaces().begin(); 
  
  while (itCoordIndex != coordIndex.end())
  {
    while ((*itCoordIndex != -1) && (itCoordIndex != coordIndex.end()))
    {
      Mesh::Vertex *V = mesh->getVertices()[*itCoordIndex];
     
      VertexFData *vfdata = new VertexFData();
  
      if (!pointArray.empty())
        V->data().setPoint(pointArray[*itCoordIndex]);
      if (!normalArray.empty())
        vfdata->setNormal(normalArray[*itNormalIndex]);
      if (!colorArray.empty())
        vfdata->setColor(colorArray[*itColorIndex]);
      if (!texCoordArray.empty())
        vfdata->setTexCoord(texCoordArray[*itTexCoordIndex]);  
      
      if (!normalPerVertex)
        (*itFace)->data().setNormal(normalArray[*itNormalIndex]);
      if (!colorPerVertex)
        (*itFace)->data().setColor(colorArray[*itColorIndex]);  
      
//      if (V->getIndex() == 0)
//        std::cout << "V 0 inserts " << (*itFace)->getIndex() << ", " << *itFace << std::endl;
        
      V->data().addFaceData(*itFace, vfdata);    
      
      ++itCoordIndex;
      if (colorPerVertex)
        ++itColorIndex;     
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itTexCoordIndex;
    }
    
    ++itCoordIndex;
    if (!colorPerVertex)
      ++itColorIndex;
    ++itNormalIndex;
    ++itTexCoordIndex;
    
    ++itFace;
  }
  
  /*Mesh::Edge *E = mesh->getEdge(mesh->getVertices()[1], mesh->getVertices()[2]);
  std::cout << "from = " << E->getFromVertex()->getIndex() << std::endl;
  std::cout << "to = " << E->getToVertex()->getIndex() << std::endl;
  
  std::cout << "leftFaces" << std::endl;
  Mesh::MFace fl = E->getLeftFaces();
  for (Mesh::MFace::const_iterator it = fl.begin(); it != fl.end(); ++it)
    std::cout << "face = " << (*it)->getIndex() << std::endl;
    
  std::cout << "RightFaces" << std::endl;
  Mesh::MFace fl2 = E->getRightFaces();
  for (Mesh::MFace::const_iterator it = fl2.begin(); it != fl2.end(); ++it)
    std::cout << "face = " << (*it)->getIndex() << std::endl;
  
  std::cout << "All Faces" << std::endl;
  Mesh::MFace fl3 = E->getFaces();
  for (Mesh::MFace::const_iterator it = fl3.begin(); it != fl3.end(); ++it)
    std::cout << "face = " << (*it)->getIndex() << std::endl;
  
  
  Mesh::Edge *E2 = E->getSymetric();
  std::cout << "from = " << E2->getFromVertex()->getIndex() << std::endl;
  std::cout << "to = " << E2->getToVertex()->getIndex() << std::endl;

  std::cout << "LeftFaces" << std::endl;
  fl = E2->getLeftFaces();
  for (Mesh::MFace::const_iterator it = fl.begin(); it != fl.end(); ++it)
    std::cout << "face = " << (*it)->getIndex() << std::endl;
  
  std::cout << "RightFaces" << std::endl;
  fl2 = E2->getRightFaces();
  for (Mesh::MFace::const_iterator it = fl2.begin(); it != fl2.end(); ++it)
    std::cout << "face = " << (*it)->getIndex() << std::endl;

  std::cout << "All Faces" << std::endl;
  fl3 = E2->getFaces();
  for (Mesh::MFace::const_iterator it = fl3.begin(); it != fl3.end(); ++it)
    std::cout << "face = " << (*it)->getIndex() << std::endl;*/

};

}
}
