/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////
//                            TemplateEdgeContent.h                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef TEMPLATEEDGECONTENT_H
#define TEMPLATEEDGECONTENT_H

#include "MeshTypes.h"

#include <vector>

namespace X3DTK {
namespace Mesh {

template<class A, class B, class C> class TemplateVertex;
template<class A, class B, class C> class TemplateFace;

/// Template class providing the Edge content for the Mesh structure. Indeed these are 
/// datas that the two symetric oriented-edges share.

template<class EData, class FData, class VData>
class TemplateEdgeContent
{
  template<class A, class B, class C, class D> friend class TemplateMesh;
  template<class A, class B, class C> friend class TemplateEdge;
public:
  /// Constructor.
  TemplateEdgeContent(TemplateVertex<VData, EData, FData> *const from, TemplateVertex<VData, EData, FData> *const to) 
  : _from(from), _to(to), _isBoundary(true)
  {};
  /// Gets the Vertex from.
  inline TemplateVertex<VData, EData, FData> *const getFromVertex() const {return _from;};
  /// Gets the Vertex to.
  inline TemplateVertex<VData, EData, FData> *const getToVertex() const {return _to;};
  /// Gets the 1-faces.
  inline const MTemplateFace<FData, VData, EData> &get1Faces() const {return _r1Faces;};
  /// Gets the 2-faces.
  inline const MTemplateFace<FData, VData, EData> &get2Faces() const {return _r2Faces;};
  /// Returns true if is a boundary.
  inline bool isBoundary() const 
  {
    return _isBoundary;
  };
  /// Gets the data.
  inline EData &data() {return _data;};
  /// Gets the data.
  inline const EData &data() const {return _data;};

private:
  TemplateVertex<VData, EData, FData> *const _from;
  TemplateVertex<VData, EData, FData> *const _to;
  MTemplateFace<FData, VData, EData> _r1Faces;
  MTemplateFace<FData, VData, EData> _r2Faces;
  bool _isBoundary;
  EData _data;
  
  /// Adds a 1-face.
  void add1Face(TemplateFace<FData, VData, EData> *face) 
  {
      _r1Faces.back() = face;
      _r1Faces.push_back((TemplateFace<FData, VData, EData> *)(0x1));
      if (_r2Faces.size() > 0)
        _isBoundary = false;
  };
  /// Adds a 2-face.
  void add2Face(TemplateFace<FData, VData, EData> *face) 
  {
      _r2Faces.back() = face;
      _r2Faces.push_back((TemplateFace<FData, VData, EData> *)(0x1));
      if (_r1Faces.size() > 0)
        _isBoundary = false;
  };

};

}
}

#endif
