#include "WorldCoordTranslatorRenderingVisitor.h"
#include "Coordinate.h"
#include "Normal.h"
#include "X3DGeometryNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {


WorldCoordTranslatorRenderingVisitor::WorldCoordTranslatorRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<WorldCoordTranslatorRenderingVisitor, Coordinate>(&WorldCoordTranslatorRenderingVisitor::enterCoordinate);
  defineNewEnterFunction<WorldCoordTranslatorRenderingVisitor, Normal>(&WorldCoordTranslatorRenderingVisitor::enterNormal);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<WorldCoordTranslatorGlobalVariables>();
}

WorldCoordTranslatorRenderingVisitor::~WorldCoordTranslatorRenderingVisitor()
{
}

void WorldCoordTranslatorRenderingVisitor::enterCoordinate(Coordinate *C) const
{
  SFMatrix34f transform = globalVariables->getMatrix();
  
  MFVec3f vertexArray = C->getPoint();
  for (MFVec3f::iterator itVertex = vertexArray.begin(); itVertex != vertexArray.end(); ++itVertex)
    *itVertex = SFVec3f(transform*SFPoint3f(*itVertex));

  C->setPoint(vertexArray);  
}

void WorldCoordTranslatorRenderingVisitor::enterNormal(Normal *N) const
{
  SFMatrix34f transform = globalVariables->getMatrix();
  
  MFVec3f normalArray = N->getVector();
  for (MFVec3f::iterator itNormal = normalArray.begin(); itNormal != normalArray.end(); ++itNormal)
  {
    *itNormal = (transform*(*itNormal));
    (*itNormal).normalize();
  }  

  N->setVector(normalArray);  
}

}
}
